###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from vistrails.db.domain import DBActionAnnotation

import unittest
import copy
import vistrails.core

class ActionAnnotation(DBActionAnnotation):

    ##########################################################################
    # Constructors and copy

    def __init__(self, *args, **kwargs):
        DBActionAnnotation.__init__(self, *args, **kwargs)
        if self.id is None:
            self.id = -1
        
    def __copy__(self):
        return ActionAnnotation.do_copy(self)

    def do_copy(self, new_ids=False, id_scope=None, id_remap=None):
        cp = DBActionAnnotation.do_copy(self, new_ids, id_scope, id_remap)
        cp.__class__ = ActionAnnotation
        return cp

    @staticmethod
    def convert(_annotation):
        _annotation.__class__ = ActionAnnotation

    ##########################################################################
    # Properties

    id = DBActionAnnotation.db_id
    key = DBActionAnnotation.db_key
    value = DBActionAnnotation.db_value
    action_id = DBActionAnnotation.db_action_id
    date = DBActionAnnotation.db_date
    user = DBActionAnnotation.db_user

    ##########################################################################
    # Operators
    
    def __str__(self):
        """__str__() -> str - Returns a string representation of an
        ActionAnnotation object.

        """
        rep = ("<actionAnnotation id=%s action_id=%s key=%s value=%s "
               "date=%s user=%s</annotation>")
        return  rep % (str(self.id), str(self.action_id), str(self.key), 
                       str(self.value), str(self.date), str(self.user))

    def __eq__(self, other):
        """ __eq__(other: ActionAnnotation) -> boolean
        Returns True if self and other have the same attributes. Used by == 
        operator. 
        
        """
        if type(self) != type(other):
            return False
        if self.key != other.key:
            return False
        if self.value != other.value:
            return False
        if self.action_id != other.action_id:
            return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

################################################################################
# Unit tests


class TestActionAnnotation(unittest.TestCase):

    def create_annotation(self, id_scope=None):
        from vistrails.db.domain import IdScope

        if id_scope is None:
            id_scope = IdScope()
        annotation = \
            ActionAnnotation(id=id_scope.getNewId(ActionAnnotation.vtType),
                             key='akey', action_id=1L,
                             value='some value', user='test')
        return annotation

    def test_copy(self):
        from vistrails.db.domain import IdScope
        id_scope = IdScope()

        a1 = self.create_annotation(id_scope)
        a2 = copy.copy(a1)
        self.assertEquals(a1, a2)
        self.assertEquals(a1.id, a2.id)
        a3 = a1.do_copy(True, id_scope, {})
        self.assertEquals(a1, a3)
        self.assertNotEquals(a1.id, a3.id)

    def test_serialization(self):
        import vistrails.core.db.io
        a1 = self.create_annotation()
        xml_str = vistrails.core.db.io.serialize(a1)
        a2 = vistrails.core.db.io.unserialize(xml_str, ActionAnnotation)
        self.assertEquals(a1, a2)
        self.assertEquals(a1.id, a2.id)

    def test_str(self):
        a1 = self.create_annotation()
        str(a1)



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2013-2014, NYU-Poly.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

from __future__ import division

from datetime import datetime
import os

from vistrails.core.modules.basic_modules import Directory, File, Path, \
    PathObject
from vistrails.core.modules.config import IPort, OPort
from vistrails.core.modules.vistrails_module import Module, ModuleError

from .common import KEY_TYPE, TYPE_CACHED, KEY_TIME, KEY_SIGNATURE, \
    get_default_store


class CachedPath(Module):
    """Uses the file store as a cache.

    Stores the path in the cache along with the module signature, so it can be
    retrieved in a later execution of the same pipeline.

    This is very similar to PersistedPath except it doesn't record more
    metadata than what's required for caching.
    """

    _input_ports = [
            IPort('path', Path)]
    _output_ports = [
            OPort('path', Path)]

    _cached = None

    def update_upstream(self):
        if not hasattr(self, 'signature'):
            raise ModuleError(self, "Module has no signature")
        file_store = get_default_store()
        entries = file_store.query({KEY_SIGNATURE: self.signature})
        best = None
        for entry in entries:
            if best is None or entry[KEY_TIME] > best[KEY_TIME]:
                best = entry
        if best is not None:
            self._cached = best.filename
        else:
            super(CachedPath, self).update_upstream()

    def compute(self):
        if self._cached is not None:
            self._set_result(self._cached)
        else:
            file_store = get_default_store()
            newpath = self.get_input('path').name
            self.check_path_type(newpath)
            metadata = {
                    KEY_TYPE: TYPE_CACHED,
                    KEY_TIME: datetime.strftime(datetime.utcnow(),
                                                '%Y-%m-%d %H:%M:%S'),
                    KEY_SIGNATURE: self.signature}
            entry = file_store.add(newpath, metadata)
            self.annotate({'added_file': entry['hash']})
            self._set_result(entry.filename)

    def check_path_type(self, path):
        pass

    def _set_result(self, path):
        self.set_output('path', PathObject(path))


class CachedFile(CachedPath):
    """Uses the file store as a cache.

    Stores the file in the cache along with the module signature, so it can be
    retrieved in a later execution of the same pipeline.

    This is very similar to PersistedFile except it doesn't record more
    metadata than what's required for caching.
    """

    _input_ports = [
            IPort('path', File)]
    _output_ports = [
            OPort('path', File)]

    def check_path_type(self, path):
        if not os.path.isfile(path):
            raise ModuleError(self, "Path is not a file")


class CachedDir(CachedPath):
    """Uses the file store as a cache.

    Stores the directory in the cache along with the module signature, so it
    can be retrieved in a later execution of the same pipeline.

    This is very similar to PersistedDir except it doesn't record more metadata
    than what's required for caching.
    """

    _input_ports = [
            IPort('path', Directory)]
    _output_ports = [
            OPort('path', Directory)]

    def check_path_type(self, path):
        if not os.path.isdir(path):
            raise ModuleError(self, "Path is not a directory")



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

from __future__ import division

import sys
import os.path
import httplib
import urllib

import vistrails.core.modules.module_registry
import vistrails.core.modules.basic_modules
from vistrails.core.modules import vistrails_module
from vistrails.core.modules.vistrails_module import Module, ModuleError, new_module
import vistrails.core.system
import enumeration_widget

import cPickle

from ZSI.ServiceProxy import ServiceProxy
from ZSI.generate.wsdl2python import WriteServiceModule
from ZSI.wstools import WSDLTools


package_directory = None
#Dictionary that contains the methods and complex types of all the web services
webServicesmodulesDict = {}
complexsdict ={} 

pm = vistrails.core.packagemanager.get_package_manager()
###############################################################################

class WebService(Module):
    """ WebService is the base Module.
     We will create a WebService Module for each method published by 
     the web service.

    """
    def __init__(self):
        Module.__init__(self)

    def compute(self):
        raise vistrails_module.IncompleteImplementation

    def runMethod(self, methodName, *args):
        return getattr(self.server,methodName)(*args)

#Class for complex types
class WBModule:
    def __init__(self,name):
        self.name = name
        self.namespace = ''
        self.webservice = ''
        self.typeobj = ''   #Enumeration, ComplexType or Method.
        self.ports = []   #Have the elements of a complex type and the elements of an enumeration.
        self.attributes = []    #Have the attributes of a complex type
        self.hasAttributes = False
        self.isRequestType = False
        self.isEmptySequence = False
        self.isExtension = False
        self.ExtensionBase = ''
        self.superClass = ''

def webServiceNameClassifier():
    """  """
    def compute(self):
        raise vistrails_module.IncompleteImplementation

    return { 'compute': compute}

def webServiceNameMethodDict():
    """ This returns the method dictionary for the web service address base
    class. """
    def compute(self):
        raise vistrails_module.IncompleteImplementation

    return { 'compute': compute}

def webServiceTypesDict(WBobj):
    """ This will create a correct compute method according to the complex type
    or enumeration name. """

    def unwrapRequestobj(self,libobj,visobj):

        #Find the children for the vistrail type and set the attributes
        #Check to see if the name of the port is repeated
        portname = visobj.name
        dictkey = WBobj.webservice + "." + portname
        complexsdict = webServicesmodulesDict[WBobj.namespace] 
        obj = complexsdict[dictkey]
        for child in obj.ports:
            namechild = child[0]
            typechild = child[1]
            nameattrib = str(namechild)
            nameattrib = nameattrib[0].upper() + nameattrib[1:]
            sentence = "visobj" + "." + nameattrib
            visdata = eval(sentence)
            if visdata is not None:
                try:
                    Type = wsdlTypesDict[str(typechild)]
                    setattr(libobj,nameattrib,visdata)
                except KeyError:
                    #Check if the port is an Enumeration Constant
                    keyvalue = WBobj.namespace.replace('|Types','') + "." + str(child[1])
                    childType = complexsdict[keyvalue]
                    if childType.typeobj == 'Enumeration':
                        visobjchild = getattr(visobj,nameattrib)
                        setattr(libobj, nameattrib, visobjchild)
                    else:
                        namemethod = "new_" + namechild
                        libobjchild = getattr(libobj,namemethod)()
                        visobjchild = getattr(visobj,nameattrib)
                        setattr(libobj,nameattrib,libobjchild)
                        unwrapRequestobj(self,libobjchild,visobjchild)
        if obj.hasAttributes:                
            for attributes in obj.attributes:
                nameattrib = str(attributes[0])
                nameattribute = nameattrib[0].upper() + nameattrib[1:]
                sentence = "visobj" + "." + nameattribute
                visdata = eval(sentence)
                if visdata is not None:
                    nameattrib = "attribute_typecode_dict[" + nameattrib + "].pname"
                    setattr(libobj,nameattrib,visdata)

    def compute(self):
        reg = vistrails.core.modules.module_registry.get_module_registry()
        #Check if it is an enumeration
        if WBobj.typeobj == 'Enumeration':
            nameport = str(WBobj.ports[0][0])
            if self.has_input(nameport):
                inputport = self.get_input(nameport)
                self.holder = inputport
                self.set_output(WBobj.name,self)
                self.set_output('self',self)
        else:
            #Check if it is a request type
            modbyname = reg.get_module_by_name(identifier = identifier, name = WBobj.name, namespace = WBobj.namespace)
            porttype = str(modbyname.server._wsdl.portTypes.keys()[0][0][1])
            listoperations = modbyname.server._wsdl.portTypes[porttype].operations.values()
            isrequest = False
            requestname = ''
            modname = WBobj.name
            for element in listoperations:
                parts = modbyname.server._wsdl.messages[element.input.getMessage().name].parts.values()
                for part in parts:
                    if str(part.element[1]).strip() == modname.strip():
                        requestname = element.input.getMessage().name
                        isrequest = True
                        break
            if (isrequest == True):
                WBobj.isRequestType = True
                #Create the request object
                try:
                    req = getattr(self.modclient,requestname)()
                except:
                    print "sys.exc_value: ", sys.exc_value
                #Set the values of the input ports in the resquest object
                for types in WBobj.ports:
                    nameport = str(types[0])
                    if self.has_input(nameport):
                        #We need to distinguish between simple and complex types.
                        inputport = self.get_input(nameport)
                        try:
                            Type = str(types[1])
                            if isArray(Type):
                                Type = 'Array'
                            Type = wsdlTypesDict[Type]
                            namemethod = "set_element_" + nameport
                            try:
                                getattr(req, namemethod)(inputport)
                            except:
                                namemethod = nameport
                                setattr(req, namemethod,inputport)
                        except KeyError:
                            keyvalue = WBobj.namespace.replace('|Types','') + "." + str(types[1])
                            #Check if the port is an Enumeration Constant
                            complexsdict = webServicesmodulesDict[WBobj.namespace]
                            childType = complexsdict[keyvalue]
                            if childType.typeobj == 'Enumeration':
                                namemethod = "set_element_" + nameport
                                getattr(req, namemethod)(inputport)
                            else:
                                namemethod = "new_" + nameport
                                libobj = getattr(req,namemethod)()
                                unwrapRequestobj(self,libobj,inputport)
                                namemethod = "set_element_" + nameport
                                getattr(req, namemethod)(libobj)
                if WBobj.hasAttributes:
                    for types in WBobj.attributes:
                        nameport = str(types[0])
                        if self.has_input(nameport):
                            inputport = self.get_input(nameport)
                            Type = wsdlTypesDict[str(types[1])]
                            nameattrib = "attribute_typecode_dict[" + nameport + "].pname"
                            setattr(req, nameattrib,inputport)

                #Set the value in the response output port
                nameport = str(WBobj.name)
                #This step is to warranty that the name are not going to repeat    
                for ports in WBobj.ports:
                     if str(WBobj.name.strip()) == str(ports[0].strip()):
                         nameport = WBobj.vistrailsname
                         break
                if WBobj.hasAttributes:
                    for attributes in WBobj.attributes:
                        if str(WBobj.name.strip()) == str(attributes[0].strip()):
                            nameport = WBobj.vistrailsname
                            break
                self.set_output(nameport,req)
                self.set_output('self',req)
            else:
                nameport = str(WBobj.name)
                for ports in WBobj.ports:
                    if str(WBobj.name.strip()) == str(ports[0].strip()):
                        nameport = WBobj.vistrailsname
                        break
                if WBobj.hasAttributes:
                    for attributes in WBobj.attributes:
                        if str(WBobj.name.strip()) == str(attributes[0].strip()):
                            nameport = WBobj.vistrailsname
                            break
                if self.has_input(nameport):
                    #Output modules
                    inputport = self.get_input(nameport)
                    for nameport in WBobj.ports:
                        nameattrib = nameport[0][0].upper() + nameport[0][1:]
                        sentence = "inputport" + "." + nameattrib
                        outputport = eval(sentence)
                        self.set_output(nameport[0],outputport)
                    if WBobj.hasAttributes:
                        for attributes in WBobj.attributes:
                            nameattrib = attributes[0][0].upper() + attributes[0][1:]
                            sentence = "inputport" + "." + nameattrib
                            outputport = eval(sentence)
                            self.set_output(attributes[0],outputport)
                elif self.has_input('self'):
                    #Now we use the 'self' input port name
                    #we keep the old for backwards compatibility
                    #Output modules
                    inputport = self.get_input('self')
                    for nameport in WBobj.ports:
                        nameattrib = nameport[0][0].upper() + nameport[0][1:]
                        sentence = "inputport" + "." + nameattrib
                        outputport = eval(sentence)
                        self.set_output(nameport[0],outputport)
                    if WBobj.hasAttributes:
                        for attributes in WBobj.attributes:
                            nameattrib = attributes[0][0].upper() + attributes[0][1:]
                            sentence = "inputport" + "." + nameattrib
                            outputport = eval(sentence)
                            self.set_output(attributes[0],outputport)
                else:    
                    #Set the values in the input ports
                    #Input modules
                    for types in WBobj.ports:
                        nameport = str(types[0])
                        nameattrib = str(nameport)
                        nameattrib = nameattrib[0].upper() + nameattrib[1:]
                        if self.has_input(nameport):
                            inputport = self.get_input(nameport)
                            setattr(self,nameattrib,inputport)
                        else:
                            setattr(self,nameattrib,None)
                    if WBobj.hasAttributes:
                        for types in WBobj.attributes:
                            nameport = str(types[0])
                            nameattrib = str(nameport)
                            nameattrib = nameattrib[0].upper() + nameattrib[1:]
                            if self.has_input(nameport):
                                inputport = self.get_input(nameport)
                                setattr(self,nameattrib,inputport)
                            else:
                                setattr(self,nameattrib,None)
                    #Set the value in the response output port
                    nameport = str(WBobj.name)
                    #This step is to warranty that the name are not going to repeat    
                    for ports in WBobj.ports:
                        if str(WBobj.name.strip()) == str(ports[0].strip()):
                            nameport = WBobj.vistrailsname
                            break
                    if WBobj.hasAttributes:
                        for attributes in WBobj.attributes:
                            if str(WBobj.name.strip()) == str(attributes[0].strip()):
                                nameport = WBobj.vistrailsname
                                break
                    self.set_output(nameport,self)
                    self.set_output('self',self)

    return {'compute':compute}


def webServiceParamsMethodDict(name, server, inparams, outparams):
    """ This will create a correct compute method according to the 
    inparams and outparams. Right now, only the first outparam is used
    for setting the result. """

    reg = vistrails.core.modules.module_registry.get_module_registry()

    def wrapResponseobj(self,resp,visobj):
        if isinstance(resp, list):
            ptype = resp[0].typecode.type[1]
        else:
            if resp.typecode.type[1] is None:
                ptype = resp.typecode.pname
            else:    
                ptype = resp.typecode.type[1]

        #Find the children for the vistrail type and set the attributes
        dictkey =  self.webservice + "|Types"        
        complexsdict = webServicesmodulesDict[dictkey]
        dictkey = self.webservice + "." + str(ptype)
        obj = complexsdict[dictkey]
        for child in obj.ports:
            namechild = child[0]
            typechild = child[1]    
            try:
                Type = wsdlTypesDict[str(typechild)]
                namemethod = "get_element_" + str(namechild)
                try:
                    ans = getattr(resp, namemethod)()
                except:
                    #Some times when there is no complex type in the wsdl
                    #the get and set methods are not generated
                    namemethod = str(namechild)
                    sentence = "resp" + "." + namemethod
                    ans = eval(sentence)
                nameattrib = str(namechild)
                nameattrib = nameattrib[0].upper() + nameattrib[1:]
                setattr(visobj,nameattrib,ans)

            except KeyError:
                if not isinstance(resp, list):
                    namemethod = "get_element_" + namechild
                    try:
                        resp = getattr(resp, namemethod)()
                    except:
                        namemethod = str(namechild)
                        sentence = "resp" + "." + namemethod
                        resp = eval(sentence)

                if isinstance(resp, list):
                    objlist = []
                    for element in resp:
                        ptype = element.typecode.type[1]
                        dictkey = self.webservice + "|Types" 
                        complexsdict = webServicesmodulesDict[dictkey]
                        dictkey = self.webservice + "." + str(ptype)
                        obj = complexsdict[dictkey]
                        visclass = reg.get_module_by_name(identifier=identifier,
                                                          name=obj.name,
                                                          namespace=obj.namespace)
                        vischildobj = visclass()
                        wrapResponseobj(self,element,vischildobj)
                        objlist.append(vischildobj)
                    nameattrib = str(namechild)
                    nameattrib = nameattrib[0].upper() + nameattrib[1:]
                    setattr(visobj,nameattrib,objlist)  
                else:
                    ptype = str(typechild)
                    dictkey = self.webservice + "|Types"
                    complexsdict = webServicesmodulesDict[dictkey]
                    dictkey = self.webservice + "." + ptype
                    obj = complexsdict[dictkey]
                    vischildclass = reg.get_module_by_name(identifier=identifier,
                                                           name=obj.name,
                                                           namespace = obj.namespace)
                    vischildobj = vischildclass()
                    nameattrib = str(namechild)
                    nameattrib = nameattrib[0].upper() + nameattrib[1:]
                    #Set the attribute in the vistrail object (Same as in
                    # schema but with the first letter uppercase)
                    setattr(visobj,nameattrib,vischildobj)  
                    visobjchild = getattr(visobj,nameattrib)
                    wrapResponseobj(self,resp,visobjchild)

    def compute(self):
        dirmodule = dir(self.modclient)
        #Get the locator name
        for elemlist in dirmodule:
            if elemlist.find('Locator') != -1:
                namelocator = elemlist
                break
        #Get the port name
        loc = getattr(self.modclient, namelocator)()
        dirloc = dir(loc)
        for elemlist in dirloc:
            if ((elemlist.find('Address') == -1) and
                ((elemlist.find('Port') != -1 and
                  elemlist.find('get') != -1) or
                 (elemlist.find('get') != -1 and
                  elemlist.find('Soap') != -1) or
                 (elemlist.find('get') != -1))):
                portname = elemlist
        port = getattr(loc, portname)()
        try:
            #This part is for the primitive or simple types methods parameters
            Type = str(inparams[0].type[1])
            if isArray(Type):
                Type = 'Array'
            Type = wsdlTypesDict[Type]
            porttype = server._wsdl.portTypes.keys()[0][0][1]
            listoperations = server._wsdl.portTypes[porttype].operations.values()
            for element in listoperations: #list of operations instances
                if str(element.name) == str(name):
                    #get the request method name
                    reqname = element.input.getMessage().name
            req = getattr(self.modclient,reqname)()
            for inparam in inparams:
                #Now set all attributes for the request object
                if self.has_input(inparam.name):
                    inputport = self.get_input(inparam.name)
                    namemethod = "set_element_" + inparam.name
                    try:
                        getattr(req, namemethod)(inputport)
                    except:
                        namemethod = inparam.name
                        setattr(req, namemethod,inputport)
            try:
                resp = getattr(port,name)(req)
            except:
                print "sys.exc_value: ", sys.exc_value
                raise ModuleError(self, str(sys.exc_value))
            namemethod = "get_element_" + outparams[0].name

            try:
                result = getattr(resp,namemethod)()
            except:
                namemethod = outparams[0].name
                sentence = "resp" + "." + namemethod
                result = eval(sentence)
            self.set_output(outparams[0].name,result)
        except KeyError:
            #This part is for the complex types methods parameters
            inparam = str(inparams[0].name)
            if self.has_input(inparam):
                request = self.get_input(inparam)
                try:
                    resp = getattr(port,name)(request)
                except:
                    print "sys.exc_value: ", sys.exc_value
                    raise ModuleError(self, str(sys.exc_value))
                #Wrap the ZSI attributes into the vistrails module attributes
                nameclass = str(outparams[0].type[1])
                dictkey = self.webservice + "|Types"
                complexsdict = webServicesmodulesDict[dictkey]
                dictkey = self.webservice + "." + nameclass
                obj = complexsdict[dictkey]
                visclass = reg.get_module_by_name(identifier=identifier,
                                                  name=obj.name,
                                                  namespace=obj.namespace)
                visobj = visclass()
                wrapResponseobj(self,resp, visobj)
                self.set_output(outparams[0].name,visobj)

    return {'compute':compute}

def processEnumeration(complexschema):
    listenum = []
    Type = complexschema.content.attributes['base'][1]
    for facets in complexschema.content.facets:
        listenum.append(facets.getAttributeDictionary()['value'])
    return (listenum,Type)

def processArray(complexschema,w):
    # the problem here is that sometimes we are told that complexschema is an
    # array but it is not. So if we get an AttributeError we will consider that
    # complexschema is a regular complex type.
    try:
        contentschema =  complexschema.content.content.attr_content
    except AttributeError:
        print "warning: type is not an array..."
        processType(complexschema,w)
        return
    
    for child in contentschema:
        nametype = str(child.attributes['http://schemas.xmlsoap.org/wsdl/']['arrayType'])
        index = nametype.find(':')
        nametype = nametype[index+1:]
        #verify if is an array of complex types
        index = nametype.find('[')
        if index != -1:
            Type = nametype[0:index]
        try:
            Type = wsdlTypesDict[str(Type)]
        except KeyError:
            try:
                complex1 = schema.types[str(Type)]
                processType(complex1,w)
            except KeyError:    
                pass

def isArray(Type):   
    #FIXME: this function not always work!!
    #Just because there's the word Array doesn't mean it will be an array (Emanuele)         
    if len(Type) > 5:
        if Type[0:5] == 'Array':
            return True
        else:
            return False

def generatename(name):
    name = name + "_1"
    return name

def processType(complexschema,w):
    
    contentschema = ''
    modulename = str(complexschema.attributes['name'])
    #print "processType: %s,%s"%(modulename,w)
    try:
        moduletype = str(complexschema.attributes['type'][1])
    except KeyError:
        moduletype = ''
    modulekey = w + "." + modulename
    objModule = WBModule(modulename)
    objModule.superClass = moduletype
    if complexschema.isSimple():
        objModule.typeobj = 'Enumeration'
        objModule.ports.append(processEnumeration(complexschema))

    if complexschema.isElement():
        try:
            if complexschema.content.content is None:
                contentschema = []
            else:    
                contentschema = complexschema.content.content.content
            objModule.typeobj = 'ComplexType'
            if (complexschema.content.content is None or
                complexschema.content.content.content == ()):
                objModule.isEmptySequence = True
        except AttributeError:
            pass

    if complexschema.isComplex():
        try:
            #Extensions in a complex type
            if complexschema.content.content.isExtension():
                objModule = WBModule(modulename)
                objModule.typeobj = 'ComplexType'
                objModule.isExtension = True
                objModule.ExtensionBase = str(complexschema.content.content.attributes['base'][1])
                contentschema = complexschema.content.content.content.content
        except AttributeError:
            contentschema = complexschema.content.content
    try:       
        #Get all the attributes of the complex type
        if complexschema.attr_content is not None:
            for attribute in complexschema.attr_content:
                nametype = attribute.getAttributeName()
                Type = 'string'
                objModule.hasAttributes = True
                objModule.attributes.append((nametype,Type))
    except AttributeError:
        pass

    #Get all the child elements of the complex type
    for child in contentschema:
        try:
            nametype = child.attributes['name']
            Type = child.attributes['type'][1]
            if isArray(Type):
                Type = 'Array'
                objModule.ports.append((nametype,Type))
                processArray(child,w)
            objModule.ports.append((nametype,Type))
            try:
                Type = wsdlTypesDict[str(Type)]
            except KeyError:
                try:
                    complex1 = schema.types[str(Type)]
                    processType(complex1,w)
                except KeyError:    
                    pass
        except AttributeError:
            pass

    objModule.webservice = str(w)
    complexsdict [modulekey] = objModule

def addExtensionsModules(w, server, extensionbase):
    """ Add complex types specified in the extension base if they are not in
    the complex types dictionary. """
    try:
        dictkey = w + "|Types"
        complexsdict = webServicesmodulesDict[dictkey]
        dictkey = w + "." + extensionbase
        extensionobj = complexsdict[dictkey]
    except KeyError:
        for schematypes in server._wsdl.types.keys():
            schema = server._wsdl.types[str(schematypes)]
            if len(schema.types.keys()) != 0:
                try:
                    extensionbasestr = str(extensionbase)
                    complex1 = schema.types[extensionbasestr]
                    processType(complex1,w)
                except KeyError:
                    pass        
            if len(schema.elements.keys()) != 0:
                try:
                    extensionbasestr = str(extensionbase)
                    complex1 = schema.elements[str(extensionbasestr)]
                    processType(complex1,w)
                except KeyError:
                    pass

def processExtension(w, server):
    dictkey = w + "|Types"
    complexsdict = webServicesmodulesDict[dictkey]
    keys = complexsdict.keys()
    keys.sort()
    for types in keys:
        obj = complexsdict[types]
        if obj.isExtension:
            #Add extensions to modulesDict dictionary
            extensionbase = obj.ExtensionBase
            addExtensionsModules(w, server, extensionbase)
            keys2 = complexsdict.keys()
            keys2.sort()
            for types2 in keys2:
                obj2 = complexsdict[types2]
                if obj2.name == extensionbase:
                    for port in obj2.ports:
                        obj.ports.append(port)
                    break

def addTypesModules(w,modclient,server):
    """ Create a VisTrails module for each complex type specified in
    webservicesmodulesDict dictionary. """
    def addObj(obj, namespace):
        if hasattr(obj,"superClass"):
            SuperType = str(obj.superClass)
        else:
            SuperType = ''
        #print "Adding obj ", obj.name, " ", SuperType
        if SuperType != '':
            try:
                if isArray(SuperType):
                    SuperType = 'Array'
                SuperType = wsdlTypesDict[SuperType]
                mt = new_module(SuperType,str(obj.name), webServiceTypesDict(obj))
                mt.name = obj.name
                mt.webservice = w
                mt.modclient = modclient
                mt.server = server
                mt.isEnumeration = False
                mt.namespace = namespace
                #print "  add to reg: ", mt.name, namespace, identifier
                reg.add_module(mt, namespace=namespace, 
                               package=identifier, package_version=version)
            except KeyError:
                try:
                    modname = SuperType
                    dictkey = w + "|Types"
                    typedict = webServicesmodulesDict[dictkey]
                    dictkey = w + "." + modname
                    typeObj = typedict[dictkey]
                    try:
                        SuperType = reg.get_module_by_name(identifier =identifier,
                                                       name=typeObj.name,
                                                       namespace = typeObj.namespace)
                        mt = new_module(SuperType,str(obj.name), webServiceTypesDict(obj))
                        mt.name = obj.name
                        mt.webservice = w
                        mt.modclient = modclient
                        mt.server = server
                        mt.isEnumeration = False
                        mt.namespace = namespace
                        #print "  add to reg: ", mt.name, namespace, identifier
                        reg.add_module(mt, namespace=namespace, 
                               package=identifier, package_version=version)
                    except:
                        addObj(typeObj,namespace)

                except KeyError:
                    pass
        else:
            mt = new_module(WebService,str(obj.name), webServiceTypesDict(obj))
            mt.name = obj.name
            mt.webservice = w
            mt.modclient = modclient
            mt.server = server
            mt.isEnumeration = False
            mt.namespace = namespace
            #print "  add to reg: ", mt.name, namespace, identifier
            reg.add_module(mt, namespace=namespace, 
                           package=identifier, package_version=version)    
             
    reg = vistrails.core.modules.module_registry.get_module_registry()
    namespace = w + "|Types"
    complexsdict = webServicesmodulesDict[namespace]
    keys = complexsdict.keys()
    keys.sort()
    for dictkey in keys:
        obj = complexsdict[dictkey]
        #print "key: ", dictkey, " obj: ", obj.name
        obj.namespace = namespace
        if obj.typeobj == 'Enumeration':
            mt = enumeration_widget.initialize(obj.name, namespace,identifier,
                                               version)
        else:
            
            addObj(obj,namespace)
            #mt = new_module(WebService,str(obj.name), webServiceTypesDict(obj))
            #mt.name = obj.name
            #mt.webservice = w
            #mt.modclient = modclient
            #mt.server = server
            #mt.isEnumeration = False
            #mt.namespace = namespace
            #reg.add_module(mt, namespace=namespace, 
            #               package=identifier, package_version=version)

def addMethodsModules(w,modclient,server):
    """ Create a VisTrails module for each complex type specified in
    webservicesmodulesDict dictionary. """

    reg = vistrails.core.modules.module_registry.get_module_registry()
    namespace = w + "|Methods"
    complexsdict = webServicesmodulesDict[namespace]
    keys = complexsdict.keys()
    keys.sort()
    for dictkey in keys:
        obj = complexsdict[dictkey]
        obj.namespace = namespace  
        mt = new_module(WebService,str(obj.name),
                        webServiceParamsMethodDict(str(obj.name), server,
                                                   obj.ports[0], obj.ports[1]))
        mt.name = obj.name
        mt.webservice = w
        mt.modclient = modclient
        mt.server = server
        mt.namespace = namespace
        reg.add_module(mt, namespace= namespace, package=identifier,
                       package_version=version)

def addPortsToTypes(w):
    """ Add input and output ports to the VisTrails complex type modules. """

    reg = vistrails.core.modules.module_registry.get_module_registry()
    dictkey = w + "|Types"
    complexsdict = webServicesmodulesDict[dictkey]
    keys = complexsdict.keys()
    for dictkey in keys:
        #Add the Ports that are of the module type
        obj = complexsdict[dictkey]
        if obj.webservice == w:
            objtype = reg.get_module_by_name(identifier=identifier,
                                             name=obj.name,
                                             namespace=obj.namespace)
            #print "Adding ports to ", obj.name
            portname = 'self'
            if obj.typeobj != 'Enumeration':
                for ports in obj.ports:
                    #This step is to warranty that the name are not going
                    #to repeat
                    #print " > ", ports
                    if str(portname.strip()) == str(ports[0].strip()):
                        obj.vistrailsname = generatename(portname)
                        portname = obj.vistrailsname
                        break    
                if obj.hasAttributes:
                    for attributes in obj.attributes:
                        #This step is to warranty that the name are not
                        #going to repeat
                        if str(portname.strip()) == str(attributes[0].strip()):
                            obj.vistrailsname = generatename(portname)
                            portname = obj.vistrailsname
                            break
            reg.add_input_port(objtype,portname,(objtype, ''))
            reg.add_output_port(objtype,portname,(objtype, ''))
            
            # this is to keep compatibility with previous versions
            # we will add as optional ports.
            reg.add_input_port(objtype,obj.name,(objtype, ''), optional=True)
            reg.add_output_port(objtype,obj.name,(objtype, ''), optional=True)
            
            if obj.typeobj != 'Enumeration':
                #Add ports according to the input and output parameters
                for ports in obj.ports:
                    try:
                        Type = wsdlTypesDict[str(ports[1])]
                    except KeyError:
                        portname = str(ports[1])
                        namespace = w + '|Types'
                        Type = reg.get_module_by_name(identifier = identifier, name =portname, namespace = namespace)
                    reg.add_input_port(objtype,str(ports[0]),(Type, ''))
                    reg.add_output_port(objtype,str(ports[0]),(Type, ''))
            if obj.hasAttributes == True:
                for attributes in obj.attributes:
                    Type = wsdlTypesDict[str(attributes[1])]
                    reg.add_input_port(objtype,str(attributes[0]),(Type, ''))
                    reg.add_output_port(objtype,str(attributes[0]),(Type, ''))

def addPortsToMethods(w):
    """ Add input and output ports to the VisTrails complex type modules. """

    reg = vistrails.core.modules.module_registry.get_module_registry()
    dictkey = w + "|Methods"
    complexsdict = webServicesmodulesDict[dictkey]
    keys = complexsdict.keys()
    for dictkey in keys:
        obj = complexsdict[dictkey]
        objtype = reg.get_module_by_name(identifier = identifier, name = obj.name, namespace = obj.namespace)
        #Add input ports
        for port in obj.ports[0]:
            try:
                nameport = str(port.name)
                Type = str(port.type[1])
                if isArray(Type):
                    Type = 'Array'
                Type = wsdlTypesDict[Type]
                reg.add_input_port(objtype,nameport,(Type, ''))
            except KeyError:
                try:
                    modname = str(port.type[1])
                    dictkey = w + "|Types"
                    typedict = webServicesmodulesDict[dictkey]
                    dictkey = w + "." + modname
                    typeObj = typedict[dictkey]
                    Type = reg.get_module_by_name(identifier=identifier,
                                                  name=typeObj.name,
                                                  namespace=typeObj.namespace)
                    reg.add_input_port(objtype,str(port.name),(Type, ''))
                except KeyError:
                    pass
        #Add output ports        
        for port in obj.ports[1]:
            try:
                nameport = str(port.name)
                Type = str(port.type[1])
                if isArray(Type):
                    Type = 'Array'
                Type = wsdlTypesDict[Type]
                reg.add_output_port(objtype,nameport,(Type, ''))
            except KeyError:
                try:
                    modname = str(port.type[1])
                    dictkey = w + "|Types"
                    typedict = webServicesmodulesDict[dictkey]
                    dictkey = w + "." + modname
                    typeObj = typedict[dictkey]
                    Type = reg.get_module_by_name(identifier=identifier,
                                                  name=typeObj.name,
                                                  namespace=typeObj.namespace)
                    reg.add_output_port(objtype,str(port.name),(Type, ''))
                except KeyError:
                    pass        

#Dictionary of primitive types
#list extracted from http://www.w3.org/2001/XMLSchema
wsdlTypesDict = { 'string' : vistrails.core.modules.basic_modules.String,
                'float': vistrails.core.modules.basic_modules.Float,
                'decimal': vistrails.core.modules.basic_modules.Float,
                'double': vistrails.core.modules.basic_modules.Float,
                'boolean': vistrails.core.modules.basic_modules.Boolean,
                'anyType': vistrails.core.modules.basic_modules.Variant,
                'anyURI': vistrails.core.modules.basic_modules.String,
                'DataHandler':vistrails.core.modules.basic_modules.String,
                'dateTime': vistrails.core.modules.basic_modules.String,
                'duration': vistrails.core.modules.basic_modules.String,
                'time': vistrails.core.modules.basic_modules.String,
                'date': vistrails.core.modules.basic_modules.String,
                'gYearMonth': vistrails.core.modules.basic_modules.String,
                'gYear': vistrails.core.modules.basic_modules.String,
                'gMonthDay': vistrails.core.modules.basic_modules.String,
                'gDay': vistrails.core.modules.basic_modules.String,
                'gMonth': vistrails.core.modules.basic_modules.String,
                'hexBinary': vistrails.core.modules.basic_modules.String,
                'base64Binary': vistrails.core.modules.basic_modules.String,
                'QName': vistrails.core.modules.basic_modules.String,
                'normalizedString': vistrails.core.modules.basic_modules.String,
                'token': vistrails.core.modules.basic_modules.String,
                'language': vistrails.core.modules.basic_modules.String,
                'IDREFS': vistrails.core.modules.basic_modules.String,
                'ENTITIES': vistrails.core.modules.basic_modules.String,
                'NMTOKEN': vistrails.core.modules.basic_modules.String,
                'NMTOKENS': vistrails.core.modules.basic_modules.String,
                'Name': vistrails.core.modules.basic_modules.String,
                'NCName': vistrails.core.modules.basic_modules.String,
                'ID': vistrails.core.modules.basic_modules.String,
                'IDREF': vistrails.core.modules.basic_modules.String,
                'ENTITY': vistrails.core.modules.basic_modules.String,
                'integer' : vistrails.core.modules.basic_modules.Integer,
                'nonPositiveInteger' : vistrails.core.modules.basic_modules.Integer,
                'negativeInteger' : vistrails.core.modules.basic_modules.Integer,
                'long' : vistrails.core.modules.basic_modules.Integer,
                'int' : vistrails.core.modules.basic_modules.Integer,
                'short' : vistrails.core.modules.basic_modules.Integer,
                'byte' : vistrails.core.modules.basic_modules.Integer,
                'nonNegativeInteger' : vistrails.core.modules.basic_modules.Integer,
                'unsignedLong' : vistrails.core.modules.basic_modules.Integer,
                'unsignedInt' : vistrails.core.modules.basic_modules.Integer,
                'unsignedShort' : vistrails.core.modules.basic_modules.Integer,
                'unsignedByte' : vistrails.core.modules.basic_modules.Integer,
                'positiveInteger' : vistrails.core.modules.basic_modules.Integer,
                'Array': vistrails.core.modules.basic_modules.List}

################################################################################
def load_wsdl_no_config(wsdlList):
    """load_wsdl_no_config(wsdlList: list of urls) -> (bool, errors)
    This loads the wsdl list without creating config files.
    Returns a tuple with two values. The first is a boolean that is True
    in case of success. The second is a list with problematic urls.
    
    """
    global complexsdict
    result = True
    not_loaded = []
    for w in wsdlList:
        # Validation if the user introduces a blank space in the list of web
        # services
        if w == '':
            continue
        complexsdict = {} #Dictionary that contains the methods or the
                          #types of a web service
        w = w.strip()
        numtimes = 0
        for element in wsdlList:
            if w == element.strip():
                numtimes = numtimes + 1
        if numtimes > 1:
            # as this is not a big problem, let's just print on the console
            print "Warning: Web service '%s' is repeated in the list and won't \
be loaded again." % w
            # this is not a problem, we will just ignore it
            continue
        try:
            s = w.split('/')
            host = s[2]
        except:
            msg = "Malformed URL."
            not_loaded.append((w,msg))
            result = False
            continue
        location = w
        try:
            reader = WSDLTools.WSDLReader()
            wsdl = reader.loadFromURL(location)
            
        except Exception, e:
            # we will not show this every time. Just in the end
            # so we can show a single message for everybody
            not_loaded.append((w,str(e)))
            result = False
            continue

        #create a directory for each webservice if it does not exist
        package_directory = os.path.join(vistrails.core.system.current_dot_vistrails(),
                                         "webServices")
        sys.path.append(package_directory)
        directoryname = urllib.quote_plus(w)
        directoryname = directoryname.replace(".","_")
        directoryname = directoryname.replace("%","_")
        directoryname = directoryname.replace("+","_")
        wsm = WriteServiceModule(wsdl)
        client_mod = wsm.getClientModuleName()

        #import the stub generated files
        try:
            importpackage = __import__(directoryname)
            modclient = getattr(importpackage,client_mod)
            server = ServiceProxy(w, pyclass=True, tracefile=sys.stdout)
        except Exception, e:
            msg =  "Couldn't load generated stub file: %s"%str(e)
            not_loaded.append((w,msg))
            result = False
            continue
        
        addTypesModules(w,modclient,server)
        addPortsToTypes(w)
        addMethodsModules(w,modclient,server)
        addPortsToMethods(w)
    return (result, not_loaded)

def load_wsdl_with_config(wsdlList):
    """load_wsdl_with_config(wsdlList: list of urls) -> (bool,list)
    This loads the wsdl list creating config files.
    Returns a tuple with two values. The first is a boolean that is True
    in case of success. The second is a list with problematic urls"""
    
    global schema
    global webServicesmodulesDict
    global complexsdict
    reg = vistrails.core.modules.module_registry.get_module_registry()
    basic = vistrails.core.modules.basic_modules
    not_loaded = []
    result = True
    for w in wsdlList:
        #Validation if the user introduces a blank space in the list of
        #web services
        if w == '':
            continue
        complexsdict = {} #This dictionary stores the complex types for
                          #a webservice w
        w = w.strip()
        #Check to see if the element is not repeated in the web services list
        numtimes = 0
        for element in wsdlList:
            if w == element.strip():
                numtimes = numtimes + 1
        if numtimes > 1:
            print "The following web service is repeated in the list and won't \
be loaded again: %s"% w
            continue
          
        #Create the stub files for the webservices that have been
        # modified since the last time they where loaded in VisTrails
        try:
            s = w.split('/')
            host = s[2]
        except:
            msg = "Malformed URL."
            not_loaded.append((w,msg))
            result = False
            continue
        location = w
        ok = 0 # Trying to load a few times
               # a couple of times, because sometimes we have timeout
        msg = ''
        while ok < 3:
            try:
                reader = WSDLTools.WSDLReader()
                wsdl = reader.loadFromURL(location)
                ok = 4
                failed = False
            except Exception, e:
                ok += 1
                failed = True
                msg = str(e)

        if failed:
            not_loaded.append((w,msg))
            result = False
            continue
        #create a directory for each webservice if it does not exist
        directoryname = urllib.quote_plus(w)
        directoryname = directoryname.replace(".","_")
        directoryname = directoryname.replace("%","_")
        directoryname = directoryname.replace("+","_")

        package_directory = os.path.join(vistrails.core.system.current_dot_vistrails(),
                                         "webServices")
        sys.path.append(package_directory)
        package_subdirectory = os.path.join(package_directory,
                                            directoryname)
        
        wsm = WriteServiceModule(wsdl)
        client_mod = wsm.getClientModuleName()
        types_mod = wsm.getTypesModuleName()
        client_file = os.path.join(package_subdirectory, '%s.py' %client_mod)
        types_file = os.path.join(package_subdirectory, '%s.py' %types_mod)
        filename = '__init__'
        init_file = os.path.join(package_subdirectory, '%s.py' %filename)
        conn = httplib.HTTPConnection(host)
        filename = '/' + '/'.join(s[3:])
        request = conn.request("GET", filename)
        response = conn.getresponse()
        if not os.path.isdir(package_subdirectory):
            try:
                print "Creating package subdirectory...",
                os.mkdir(package_subdirectory)
                print "done."
            except:
                msg = "\nCould not create package subdirectory. Make sure \
'%s' does not exist and parent directory is writable." % package_subdirectory
                print "Skipping '%s'" % w
                # We don't need to exit here. Continuing try to load
                # the other files
                not_loaded.append((w,msg))
                result = False
                continue
                
        #generate the stub files
        try:
            from ZSI.generate.containers import ServiceHeaderContainer,\
                 TypecodeContainerBase, TypesHeaderContainer
            kwargs = {'module':'ZSI.generate.pyclass', 
                      'metaclass':'pyclass_type'}
            TypecodeContainerBase.metaclass = kwargs['metaclass']
            TypesHeaderContainer.imports.append(\
                  'from %(module)s import %(metaclass)s' %kwargs
                  )
            ServiceHeaderContainer.imports.append (\
            'from %(module)s import %(metaclass)s' %kwargs
            )
            
            fd = open(client_file, 'w+')
            wsm.writeClient(fd)
            fd.close()
            
            fd = open(types_file, 'w+' )
            wsm.writeTypes(fd)
            fd.close()
              
            fd = open(init_file, 'w')
            content = 'import ' + client_mod
            fd.write(content)
            fd.close()
        except Exception, e:
            msg = "Couldn't generate stub file: %s"% str(e)
            not_loaded.append((w,msg))
            result = False
            continue
        #import the stub generated files
        try:
            importpackage = __import__(directoryname)
            modclient = getattr(importpackage,client_mod)
            server = ServiceProxy(w, pyclass=True, tracefile=sys.stdout)
        except Exception, e:
            msg = "Problem importing the generated stub files: %s", str(e)
            not_loaded.append((w,msg))
            result = False
            continue

        #Set up the dictionary with all the complex types modules and their ports.
        for schematypes in server._wsdl.types.keys():
            schema = server._wsdl.types[str(schematypes)]
            if len(schema.types.keys()) != 0:
                for schematype in schema.types.keys():
                    try:
                        complex1 = schema.types[str(schematype)]
                        modulename = str(complex1.attributes['name'])
                        try:
                            dictkey = w + "." + modulename
                            typeObj = complexsdict[dictkey]
                        except KeyError:
                            #Check if it is an array.  Arrays are python
                            #lists, not complex types though they can
                            #contain complex types elements.
                            Type = str(schematype)
                            if isArray(Type):
                                processArray(complex1,w)
                            else:
                                processType(complex1,w)
                    except KeyError:
                        pass
            if len(schema.elements.keys()) != 0:
                for schematype in schema.elements.keys():
                    try:
                        complex1 = schema.elements[str(schematype)]
                        modulename = str(complex1.attributes['name'])
                        try:
                            dictkey = w + "." + modulename
                            typeObj = complexsdict[dictkey]
                        except KeyError:
                            #Check if it is an array.  Arrays are python
                            #lists, not complex types, though they can
                            #contain complex types elements.
                            Type = str(schematype)
                            if isArray(Type):
                                processArray(complex1,w)
                            else:
                                processType(complex1,w)
                    except KeyError:
                        pass
        dictkey = w + "|Types"
        webServicesmodulesDict[dictkey] = complexsdict            
        complexsdict = {}
        #get all the service's methods and for each method create a
        #module also create
        # a module for the complex types classes and enumerations.
        
        # Add the Extensions if there is any in the complex types
        processExtension(w,server)

        #Create the VisTrails modules with their ports
        addTypesModules(w,modclient,server)

        addPortsToTypes(w)

        #Open the file with methods
        #Add the Methods
        keys = server._methods.keys()
        keys.sort()
        
        for kw in keys:
            methods = server._methods[kw]
            callInfo = methods[0].callinfo
            inparams = callInfo.inparams
            outparams = callInfo.outparams
            mt = new_module(WebService,str(kw), 
                            webServiceParamsMethodDict(str(kw),server,
                                                       inparams,
                                                       outparams))
            mt.webservice = w
            mt.modclient = modclient
            namespace = w + '|Methods'
            mt.namespace = namespace
            
            reg.add_module(mt, namespace= namespace, package=identifier,
                           package_version=version)
            
            objModule = WBModule(kw)
            objModule.namespace = namespace
            objModule.webservice = w
            objModule.typeobj = 'Method'
            objModule.ports.append(inparams)
            objModule.ports.append(outparams)
            #Add the input (ports) parameters to the methods
            for p in inparams:
                try:
                    nameport = str(p.name)
                    Type = str(p.type[1])
                    if isArray(Type):
                        Type = 'Array'
                    Type = wsdlTypesDict[Type]
                    reg.add_input_port(mt,nameport,(Type, ''))
                except KeyError:
                    try:
                        modname = str(p.type[1])
                        dictkey = w + "|Types"
                        typedict = webServicesmodulesDict[dictkey]
                        dictkey = w + "." + modname
                        typeObj = typedict[dictkey]
                        Type = reg.get_module_by_name(identifier =identifier,
                                                      name=typeObj.name,
                                                      namespace = typeObj.namespace)
                        reg.add_input_port(mt,str(p.name),(Type, ''))
                    except KeyError:
                        pass
            #Add the output (ports) parameters to the methods
            for p in outparams:
                try:
                    nameport = str(p.name)
                    Type = str(p.type[1])
                    if isArray(Type):
                        Type = 'Array'
                    Type = wsdlTypesDict[Type]
                    reg.add_output_port(mt,nameport,(Type, ''))
                except KeyError:
                    try:
                        modname = str(p.type[1])
                        dictkey = w + "|Types"
                        typedict = webServicesmodulesDict[dictkey]
                        dictkey = w + "." + modname
                        typeObj = typedict[dictkey]
                        Type = reg.get_module_by_name(identifier=identifier,
                                                      name=typeObj.name,
                                                      namespace = typeObj.namespace)
                        reg.add_output_port(mt,str(p.name),(Type, ''))
                    except KeyError:
                        pass
            modulekey = w + "." + str(kw)
            complexsdict[modulekey] = objModule
        dictkey = w + "|Methods"
        webServicesmodulesDict[dictkey] = complexsdict        
    #Write modules.conf file that will contain the types and methods
    #dictionary of the web services.
    namefile = os.path.join(vistrails.core.system.current_dot_vistrails(),
                            'webServices',
                            'modules.conf')    
    try:
        ouf = open(namefile, 'w')
        cPickle.dump(webServicesmodulesDict,ouf)
        ouf.close()
    except Exception, e:
        msg = "Error generating web services configuration file %s"%str(e)
        raise RuntimeError(msg)

    return(result,not_loaded)
        
def verify_wsdl(wsdlList):
    """verify_wsdl(wsdlList: list of urls) -> (list,list,list)

    This checks for the wsdls that need to be updated or the files need to be
    generated and splits them in 3 lists: files that are outdated, updated and
    ones that an error was generated.
    
    """
    outdated_list = []
    updated_list = []
    error_list = []
    for w in wsdlList:
        if w == '':
            continue
        try:
            s = w.split('/')
            host = s[2]
        except:
            msg = "Malformed URL."
            error_list.append((w,msg))
            continue
        location = w
        reader = WSDLTools.WSDLReader()
        load = reader.loadFromURL
        try:
            wsdl = load(location)
        except Exception, e:
            msg = "Couldn't load wsdl from the web: %s."%str(e)
            error_list.append((w,msg))
            continue
        directoryname = urllib.quote_plus(w)
        directoryname = directoryname.replace(".","_")
        directoryname = directoryname.replace("%","_")
        directoryname = directoryname.replace("+","_")
        package_subdirectory = os.path.join(vistrails.core.system.current_dot_vistrails(),
                                            "webServices",
                                            directoryname)
        wsm = WriteServiceModule(wsdl)
        client_mod = wsm.getClientModuleName()
        client_file = os.path.join(package_subdirectory, '%s.py' %client_mod)
        conn = httplib.HTTPConnection(host)
        filename = '/' + '/'.join(s[3:])
        request = conn.request("GET", filename)
        response = conn.getresponse()
        remoteHeader = response.msg.getheader('last-modified')
        isoutdated = False
        if remoteHeader is not None:
            localFile = client_file
            reg = vistrails.core.modules.module_registry.get_module_registry()
            httpfile = reg.get_descriptor_by_name(
                'org.vistrails.vistrails.url', 'DownloadFile').module()
            try:
                isoutdated = httpfile._is_outdated(remoteHeader, localFile)
            except OSError:
                print "File doesn't exist"
                isoutdated = True
        if isoutdated or remoteHeader is None:
            outdated_list.append(w)
        else:
            updated_list.append(w)
    return (outdated_list,updated_list, error_list)
            
def initialize(*args, **keywords):
    import vistrails.core.packagemanager
    global schema
    global webServicesmodulesDict
    global complexsdict
    wsdlList = []
    if configuration.showWarning:
        msg = ("The Web Services package is deprecated and will be removed "
               "from the next VisTrails release. Please consider using the "
               "new SUDS Web Services package.")
        pm.show_error_message(pm.get_package(identifier), msg)
    if configuration.check('wsdlList'):
        wsdlList = configuration.wsdlList.split(";")

    reg = vistrails.core.modules.module_registry.get_module_registry()
    basic = vistrails.core.modules.basic_modules
    reg.add_module(WebService)

    #Create a directory for the webServices package
    global package_directory
    package_directory = os.path.join(vistrails.core.system.current_dot_vistrails(),
                                     "webServices")
    if not os.path.isdir(package_directory):
        try:
            print "Creating package directory..."
            os.mkdir(package_directory)
        except:
            print "Could not create package directory. Make sure"
            print "'%s' does not exist and parent directory is writable"
            sys.exit(1)

    pathfile = os.path.join(vistrails.core.system.current_dot_vistrails(),
                            "webServices",
                            "modules.conf")
    outdated_list = []
    updated_list = []
    error_list = []
    if os.path.isfile(pathfile):
        #Verify if there is a need to update the modules configuration
        #file (modules.conf)
         (outdated_list, updated_list, error_list) = verify_wsdl(wsdlList)
             
    else:
        #If the modules configuration file doesn't exist, create it
        outdated_list = wsdlList 

        #If the stub files are not updated or there is not information in
        # the header about the modification date of the web service, the
        # stubs files and a modules configuration file will be created
        # otherwise the information of the modules will be obtained from
        # the modules.conf files that contains serialized data of the methods
        # and the complex types of the web services
    if os.path.isfile(pathfile):
        try: 
            inf = open(pathfile)
            webServicesmodulesDict = cPickle.load(inf)
            inf.close()
        except Exception, e:
            msg = "Error loading configuration file: ", pathfile
            raise IOError(msg)
    
    #print wsdlList, outdated_list, updated_list
    (res, not_loaded) = load_wsdl_no_config(updated_list)
    if not res:
        #there was a problem when trying to load the stubs
        # let's try creating them again
        for (w,e) in not_loaded:
            outdated_list.append(w)
        print "There were problems loading the webservices %s" % not_loaded
        print "We will try to load them again..."
    (res, not_loaded) = load_wsdl_with_config(outdated_list)
    if not res or len(error_list) > 0:
        msg = """ There were problems loading the webservices.
The following could not be loaded:\n"""
        error_list.extend(not_loaded)
        for (w,e) in error_list:
            msg += "Url: '%s', error: '%s'\n"%(w,e)
        pm.show_error_message(pm.get_package(identifier),msg)

def handle_missing_module(*args, **kwargs):
    global webServicesmodulesDict
    
    #this is the order the arguments are passed to the function
    controller = args[0]
    m_id = args[1]
    pipeline = args[2]
    
    def get_wsdl_from_namespace(m_namespace):
        try:
            wsdl = m_namespace.split("|")
            return wsdl[0]
        except:
            print "invalid namespace"
            return None
    m = pipeline.modules[m_id]
    m_namespace = m.namespace
    
    wsdl = get_wsdl_from_namespace(m_namespace)
    if wsdl:
        outdated_list = []
        updated_list = []
        error_list = []
        print "Downloading %s and adding to the Module list..."%wsdl
        pathfile = os.path.join(vistrails.core.system.current_dot_vistrails(),
                                "webServices",
                                "modules.conf")
        if os.path.isfile(pathfile):
            #Verify if there is a need to update the modules configuration
            #file (modules.conf)
            (outdated_list, updated_list, error_list) = verify_wsdl([wsdl])
            #print "verified: createconfig file is %s"%createconfigfile
        else:
            #If the modules configuration file doesn't exist, create it
            outdated_list = [wsdl]
        
        #If the stub files are not updated or there is not information in
        # the header about the modification date of the web service, the
        # stubs files and a modules configuration file will be created
        # otherwise the information of the modules will be obtained from
        # the modules.conf files that contains serialized data of the methods
        # and the complex types of the web services
        # print outdated_list, updated_list, error_list
        if os.path.isfile(pathfile):
            try: 
                inf = open(pathfile)
                webServicesmodulesDict = cPickle.load(inf)
                inf.close()
            except:
                print "Error loading configuration file"
                return False
        try:
            (res,not_loaded) = load_wsdl_no_config(updated_list)
            #print "done loading_no_config"
            if not res:
                outdated_list.extend([wsdl])
        
            (res, not_loaded) = load_wsdl_with_config(outdated_list)
            #print "done loading_with_config"
            if res:
                #add new url to package config file
                wsdlList = []
                if configuration.check('wsdlList'):
                    wsdlList = configuration.wsdlList.split(";")
                if wsdl not in wsdlList:
                    wsdlList.append(wsdl)    
                swsdlList = ";".join(wsdlList)
                configuration.wsdlList = swsdlList
                print "done."
                return True
            else:
                msg = """ There were problems loading the webservice.
The following could not be loaded:\n"""
                error_list.extend(not_loaded)
                for (w,e) in error_list:
                    msg += "Url: '%s', error: '%s'\n"%(w,e)
                    pm.show_error_message(pm.get_package(identifier),msg)
        except Exception, e:
            print e
            import traceback
            traceback.print_stack()
    print "An error occurred. Could not add missing wsdl."
    return False



#!/usr/bin/env python
###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

##############################################################################
# create_module_appearance.py helps VisTrails package developers customize
# the appearance of modules in their packages.

from PyQt4 import QtGui, QtCore, Qt
from module_appearance import Ui_MainWindow
import sys

default_pen = QtGui.QPen()
default_pen.setWidth(2)
default_pen.setColor(QtCore.Qt.black)

selected_pen = QtGui.QPen()
selected_pen.setWidth(3)
selected_pen.setColor(QtCore.Qt.yellow)

##############################################################################

class ModuleFringeJoint(QtGui.QGraphicsEllipseItem):
    
    brush = QtGui.QBrush(QtGui.QColor(192, 192, 192))
    def __init__(self, leftLine, rightLine):
        pt = rightLine.line().p1()
        sz = 5
        QtGui.QGraphicsEllipseItem.__init__(self, -sz/2, -sz/2, sz, sz)
        self.setPos(pt)
        self.leftLine = leftLine
        self.rightLine = rightLine
        self.setAcceptHoverEvents(True)
        self.setPen(default_pen)
        self.setBrush(self.brush)
        self.setZValue(1.0)
        self.setFlag(QtGui.QGraphicsItem.ItemIsMovable, True)
        self.setFlag(QtGui.QGraphicsItem.ItemIsFocusable, True)
        
    def removeSelf(self):
        scene = self.scene()
        scene.removeItem(self)
        scene.removeItem(self.rightLine)
        self.leftLine.rightJoint = self.rightLine.rightJoint
        self.leftLine.moveRightPoint(self.rightLine.line().p2())
        if self.rightLine.rightJoint is not None:
            self.rightLine.rightJoint.leftLine = self.leftLine
        
    def mouseMoveEvent(self, event):
        QtGui.QGraphicsItem.mouseMoveEvent(self, event)
        self.leftLine.moveRightPoint(self.scenePos())
        self.rightLine.moveLeftPoint(self.scenePos())
        app.window.update_text_view()

    def keyPressEvent(self, event):
        if event.matches(QtGui.QKeySequence.Delete):
            self.removeSelf()

    def hoverEnterEvent(self, event):
        self.setPen(selected_pen)
        self.setFocus(QtCore.Qt.OtherFocusReason)

    def hoverLeaveEvent(self, event):
        self.setPen(default_pen)
        self.clearFocus()

    @staticmethod
    def toScene(pt, side):
        if side == 'left':
            raise Exception("unimplemented")
        elif side == 'right':
            return QtCore.QPointF(pt[0] * 80.0 + 100.0, pt[1] * 80.0 - 40.0)
        else:
            raise Exception("side must be either 'right' or 'left'")
        
    def toVisTrails(self, side):
        px, py = self.scenePos().x(), self.scenePos().y()
        if side == 'left':
            return ((100.0+px)/80.0, (40.0-py)/80.0)
        elif side == 'right':
            return ((px-100.0)/80.0, (40.0-py)/80.0)
        else:
            raise Exception("side must be either 'right' or 'left'")

class ModuleFringeLine(QtGui.QGraphicsLineItem):

    def __init__(self, leftJoint, rightJoint, *args, **kwargs):
        QtGui.QGraphicsLineItem.__init__(self, *args, **kwargs)
        self.setAcceptHoverEvents(True)
        self.setPen(default_pen)
        self.setAcceptedMouseButtons(QtCore.Qt.LeftButton)
        self.leftJoint = leftJoint
        self.rightJoint = rightJoint
        self.setZValue(0.0)

    def mouseDoubleClickEvent(self, event):
        self.createNewLine(event.pos())
        app.window.update_text_view()

    def hoverEnterEvent(self, event):
        self.setPen(selected_pen)

    def hoverLeaveEvent(self, event):
        self.setPen(default_pen)

    def moveLeftPoint(self, pt):
        self.setLine(QtCore.QLineF(pt, self.line().p2()))

    def moveRightPoint(self, pt):
        self.setLine(QtCore.QLineF(self.line().p1(), pt))

    def createNewLine(self, pt):
        old_line = self.line()
        self.setLine(QtCore.QLineF(old_line.p1(), pt))
        new_line = QtCore.QLineF(pt, old_line.p2())
        
        old_joint = self.rightJoint

        new_fringe_line = ModuleFringeLine(None, self.rightJoint, new_line)
        new_joint = ModuleFringeJoint(self, new_fringe_line)

        self.rightJoint = new_joint
        new_fringe_line.leftJoint = new_joint
        if old_joint is not None:
            old_joint.leftLine = new_fringe_line
        
        scene = self.scene()
        scene.addItem(new_fringe_line)
        scene.addItem(new_joint)
        return new_fringe_line

##############################################################################

class MainWindow(QtGui.QMainWindow):

    def __init__(self, parent=None):
        QtGui.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.gv = self.ui.graphicsView
        self.setup_graphics_view()
        self.connect(self.ui.pushButton_Quit,
                     QtCore.SIGNAL("clicked()"),
                     QtGui.qApp.quit)
        self.connect(self.ui.pushButton_symmetric,
                     QtCore.SIGNAL("clicked()"),
                     self.make_symmetric)
        self.connect(self.ui.pushButton_mirrored,
                     QtCore.SIGNAL("clicked()"),
                     self.make_mirrored)
        self.connect(self.ui.pushButton_Clear,
                     QtCore.SIGNAL("clicked()"),
                     self.clear_points)

    def setup_graphics_view(self):
        self.scene = QtGui.QGraphicsScene()
        self.gv.setScene(self.scene)
        self.gv.setBackgroundBrush(QtCore.Qt.gray)
        pen = QtGui.QPen()
        pen.setWidth(2)
        pen.setColor(QtCore.Qt.black)

        self.scene.addLine(-100,-40,100,-40,pen)
        self.scene.addLine(-100, 40,100, 40,pen)

        # l1 is the left line
        self.l1 = ModuleFringeLine(None, None, -100,  40, -100, -40)

        # l2 is the right line
        self.l2 = ModuleFringeLine(None, None,  100, -40,  100,  40)
        
        self.scene.addItem(self.l1)
        self.scene.addItem(self.l2)

        self.update_text_view()

    @staticmethod
    def get_points(line, side):
        lst = []
        while not (line.rightJoint is None):
            lst.append(line.rightJoint.toVisTrails(side))
            line = line.rightJoint.rightLine
        return lst
    
    def update_text_view(self):
        s = ''
        left_list = self.get_points(self.l1, 'left')
        right_list = self.get_points(self.l2, 'right')
        
        for (preamble, postamble,
             side, pts) in zip(['reg.add_module(your_class,\n               moduleLeftFringe=[(0.0, 0.0), ',
                                '               moduleRightFringe=[(0.0, 0.0), '],
                               ['(0.0, 1.0)],\n', '(0.0, 1.0)])\n'],
                               ['left', 'right'],
                               [left_list, reversed(right_list)]):
            f = pts
            s += preamble
            for p in pts:
                s += ('(%.4f, %.4f), ' % p)
            s += postamble
        self.ui.textEdit.clear()
        self.ui.textEdit.append(s)

    ##########################################################################
    # slots

    def make_symmetric(self):
        while not (self.l2.rightJoint is None):
            self.l2.rightJoint.removeSelf()
        l1 = self.l1
        l2 = self.l2
        while not (l1.rightJoint is None):
            t = l1.rightJoint.toVisTrails('left')
            t = (-t[0], t[1])
            p = ModuleFringeJoint.toScene(t, 'right')
            l2 = l2.createNewLine(p)
            l1 = l1.rightJoint.rightLine
        self.update_text_view()

    def make_mirrored(self):
        while not (self.l2.rightJoint is None):
            self.l2.rightJoint.removeSelf()
        l1 = self.l1
        pl = []
        while not (l1.rightJoint is None):
            p = l1.rightJoint.scenePos()
            pl.append(QtCore.QPointF(-p.x(), p.y()))
            l1 = l1.rightJoint.rightLine
        l2 = self.l2
        for p in reversed(pl):
            l2 = l2.createNewLine(p)
        self.update_text_view()

    def clear_points(self):
        while not (self.l2.rightJoint is None):
            self.l2.rightJoint.removeSelf()
        while not (self.l1.rightJoint is None):
            self.l1.rightJoint.removeSelf()
        self.update_text_view()

##############################################################################

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    app.window = MainWindow()
    app.window.show()
    app.exec_()




###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

import copy
from vistrails.db.versions.v1_0_2.domain import DBVistrail, DBWorkflow, DBLog, \
    DBRegistry, DBGroup, DBActionAnnotation, DBAnnotation, DBAction, IdScope

def translateVistrail(_vistrail):
    tag_annotations = []
    notes_annotations = []
    thumb_annotations = []
    upgrade_annotations = []
    prune_annotations = []

    def update_type(old_obj, translate_dict):
        if old_obj.db_type.find('|') >= 0:
            [identifier, m_and_ns] = old_obj.db_type.split(':', 1)
            # this second check is fragile but should be ok
            if m_and_ns.find(':') == -1 or m_and_ns.startswith('http://'):
                # need to move module name
                try:
                    [namespace, module] = m_and_ns.rsplit('|', 1)
                    new_type = ':'.join([identifier, module, namespace])
                    return new_type
                except Exception, e:
                    # just bail for now
                    print e
                    pass
        return old_obj.db_type

    def update_tags(old_obj, translate_dict):
        for tag in old_obj.db_tags:
            tag_annotations.append((tag.db_id, tag.db_name, None))
        return []

    def update_actions(old_obj, translate_dict):
        new_actions = []
        for action in old_obj.db_actions:
            if action.db_prune == 1:
                prune_annotations.append((action.db_id, str(True), None))
            new_actions.append(DBAction.update_version(action, translate_dict))
        return new_actions

    def update_annotations(old_obj, translate_dict):
        same_annotations = []
        for annotation in old_obj.db_annotations:
            if annotation.db_key == '__notes__':
                notes_annotations.append((old_obj.db_id, annotation.db_value,
                                          annotation.db_id))
            elif annotation.db_key == '__thumb__':
                thumb_annotations.append((old_obj.db_id, annotation.db_value,
                                          annotation.db_id))
            elif annotation.db_key == '__upgrade__':
                upgrade_annotations.append((old_obj.db_id, 
                                            annotation.db_value,
                                            annotation.db_id))
            else:
                same_annotations.append(
                    DBAnnotation.update_version(annotation, translate_dict))
        return same_annotations

    def update_workflow(old_obj, translate_dict):
        return DBWorkflow.update_version(old_obj.db_workflow, translate_dict)

    translate_dict = {'DBGroup': {'workflow': update_workflow},
                      'DBVistrail': {'tags': update_tags,
                                     'actions': update_actions},
                      'DBAction': {'annotations': update_annotations},
                      'DBParameter': {'type': update_type},
                      }
    _vistrail.update_id_scope()
    vistrail = DBVistrail.update_version(_vistrail, translate_dict)

    id_scope = vistrail.idScope
    id_scope.setBeginId('annotation', _vistrail.idScope.getNewId('annotation'))
    key_lists = {'__tag__': tag_annotations,
                 '__notes__': notes_annotations,
                 '__thumb__': thumb_annotations,
                 '__upgrade__': upgrade_annotations,
                 '__prune__': prune_annotations}
    for key, annotations in key_lists.iteritems():
        for action_id, value, new_id in annotations:
            if new_id is None:
                new_id = id_scope.getNewId(DBActionAnnotation.vtType)
            annotation = DBActionAnnotation(id=new_id,
                                            action_id=action_id,
                                            key=key,
                                            value=value)
            annotation.is_new = False
            annotation.is_dirty = False
            vistrail.db_add_actionAnnotation(annotation)

    vistrail.db_version = '1.0.2'
    return vistrail

def translateWorkflow(_workflow):
    def update_workflow(old_obj, translate_dict):
        return DBWorkflow.update_version(old_obj.db_workflow, translate_dict)
    translate_dict = {'DBGroup': {'workflow': update_workflow}}
    workflow = DBWorkflow.update_version(_workflow, translate_dict)
    workflow.db_version = '1.0.2'
    return workflow

def translateLog(_log):
    translate_dict = {}
    log = DBLog.update_version(_log, translate_dict)
    log.db_version = '1.0.2'
    return log

def translateRegistry(_registry):
    translate_dict = {}
    registry = DBRegistry.update_version(_registry, translate_dict)
    registry.db_version = '1.0.2'
    return registry



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

from __future__ import division

from vistrails.core.modules import basic_modules
from vistrails.core.modules.vistrails_module import Module, ModuleError
from vistrails.core.packagemanager import get_package_manager
from PyQt4 import QtGui


class Dialog(Module):
    _input_ports = [('title', basic_modules.String,
                     {'optional': True}),
                    ('cacheable', basic_modules.Boolean,
                     {'optional': True, 'defaults': "['False']"})]

    cacheable_dialog = False

    def is_cacheable(self):
        return self.cacheable_dialog


class TextDialog(Dialog):
    _input_ports = [('label', basic_modules.String,
                     {'optional': True, 'defaults': "['']"}),
                    ('default', basic_modules.String,
                     {'optional': True, 'defaults': "['']"})]
    _output_ports = [('result', basic_modules.String)]

    mode = QtGui.QLineEdit.Normal

    def compute(self):
        if self.has_input('title'):
            title = self.get_input('title')
        else:
            title = 'VisTrails Dialog'
        label = self.get_input('label')

        default = self.get_input('default')

        self.cacheable_dialog = self.get_input('cacheable')

        (result, ok) = QtGui.QInputDialog.getText(None, title, label,
                                                  self.mode,
                                                  default)
        if not ok:
            raise ModuleError(self, "Canceled")
        self.set_output('result', str(result))


class PasswordDialog(TextDialog):
    _input_ports = [('label', basic_modules.String,
                     {'optional': True, 'defaults': "['Password']"})]

    mode = QtGui.QLineEdit.Password


class YesNoDialog(Dialog):
    _input_ports = [('label', basic_modules.String,
                     {'optional': True, 'defaults': "['Yes/No?']"})]
    _output_ports = [('result', basic_modules.Boolean)]

    def compute(self):
        if self.has_input('title'):
            title = self.get_input('title')
        else:
            title = 'VisTrails Dialog'
        label = self.get_input('label')

        self.cacheable_dialog = self.get_input('cacheable')

        result = QtGui.QMessageBox.question(
                None,
                title, label,
                QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        result = result == QtGui.QMessageBox.Yes

        self.set_output('result', result)


_modules = [(Dialog, {'abstract': True}),
            TextDialog, PasswordDialog,
            YesNoDialog]


pm = get_package_manager()
if pm.has_package('org.vistrails.vistrails.spreadsheet'):
    from .continue_prompt import _modules as continue_modules
    _modules.extend(continue_modules)



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2013-2014, NYU-Poly.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

from __future__ import division

from PyQt4 import QtCore, QtGui

import file_archive.viewer
from file_archive.viewer import StoreViewerWindow

from vistrails.core.application import get_vistrails_application
from vistrails.core.db.locator import FileLocator

from .common import get_default_store, KEY_WORKFLOW, KEY_MODULE_ID


class VistrailsViewerWindow(StoreViewerWindow):
    WINDOW_TITLE = "Persistent archive viewer"

    _vt_only = True

    def _create_buttons(self):
        buttons = super(VistrailsViewerWindow, self)._create_buttons()

        open_vt_button = QtGui.QPushButton("Go to Vistrail")
        self.connect(open_vt_button, QtCore.SIGNAL('clicked()'),
                     self._open_vt)
        buttons.append(('vt', open_vt_button))

        only_vt_checkbox = QtGui.QCheckBox("Only VisTrails Persisted files")
        only_vt_checkbox.setChecked(self._vt_only)
        self.connect(only_vt_checkbox, QtCore.SIGNAL('stateChanged(int)'),
                     self._set_vt_only)
        buttons.append(('alwayson', only_vt_checkbox))

        return buttons

    def _set_vt_only(self, state):
        state = state == QtCore.Qt.Checked
        if state == self._vt_only:
            return
        self._vt_only = state
        self._search()

    def _alter_search_conditions(self, conditions):
        if self._vt_only and not KEY_WORKFLOW in conditions:
            conditions[KEY_WORKFLOW] = {'type': 'str'}
        return conditions

    def _selection_changed(self):
        super(VistrailsViewerWindow, self)._selection_changed()
        items = self._result_tree.selectedItems()
        for t, button in self._buttons:
            if t == 'vt':
                button.setEnabled(
                        len(items) == 1 and
                        self._find_vt(items[0]) is not None)

    @staticmethod
    def _find_vt(item):
        if isinstance(item, file_archive.viewer.MetadataItem):
            item = item.parent()
        metadata = item.entry.metadata
        try:
            workflow = metadata[KEY_WORKFLOW]
            module_id = metadata[KEY_MODULE_ID]
        except KeyError:
            return None
        return (workflow, module_id)

    def _open_vt(self):
        items = self._result_tree.selectedItems()
        item = items[0]
        vt = self._find_vt(item)
        if vt is None:
            return
        workflow, module_id = vt

        if ':' in workflow:
            filename, version = workflow.rsplit(':', 1)
            try:
                version = int(version)
            except ValueError:
                filename, version = workflow, None
        else:
            filename, version = workflow, None

        app = get_vistrails_application()
        if (not app.is_running_gui() or not hasattr(app, 'builderWindow') or
                app.builderWindow is None):
            return

        view = app.builderWindow.open_vistrail(FileLocator(filename),
                                               version=version)
        if module_id is not None:
            from vistrails.gui.pipeline_view import QGraphicsModuleItem

            scene = view.controller.current_pipeline_view.scene()
            for module_item in (i for i in scene.items()
                                  if isinstance(i, QGraphicsModuleItem)):
                if module_item.module.id == module_id:
                    module_item.setSelected(True)
                    break


store = get_default_store()
viewer = VistrailsViewerWindow(store)


def show_viewer():
    viewer.show()



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

"""URL provides modules to download files via the network.

It can refer to HTTP and FTP files, which enables workflows to be distributed
without its associated data.

This package uses a local cache, inside the per-user VisTrails directory. This
way, files that haven't been changed do not need to be downloaded again. The
check is performed efficiently using HTTP headers.
"""

from __future__ import division

from datetime import datetime
import email.utils
import hashlib
import os
import re
import urllib
import urllib2

from vistrails.core.bundles.pyimport import py_import
from vistrails.core.configuration import get_vistrails_persistent_configuration
from vistrails.core import debug
import vistrails.core.modules.basic_modules
from vistrails.core.modules.basic_modules import PathObject
import vistrails.core.modules.module_registry
from vistrails.core.modules.vistrails_module import Module, ModuleError
from vistrails.core.system import current_dot_vistrails, strptime
from vistrails.core.upgradeworkflow import UpgradeWorkflowHandler
import vistrails.gui.repository
from vistrails.gui.utils import show_warning

from vistrails.core.repository.poster.encode import multipart_encode
from vistrails.core.repository.poster.streaminghttp import register_openers

from .identifiers import identifier
from .http_directory import download_directory
from .https_if_available import build_opener


package_directory = None


###############################################################################

class Downloader(object):
    def __init__(self, url, module, insecure):
        self.url = url
        self.module = module
        self.opener = build_opener(insecure=insecure)

    def execute(self):
        """ Tries to download a file from url.

        Returns the path to the local file.
        """
        self.local_filename = os.path.join(package_directory,
                                           urllib.quote_plus(self.url))

        # Before download
        self.pre_download()

        # Send request
        try:
            response = self.send_request()
        except urllib2.URLError, e:
            if self.is_in_local_cache:
                debug.warning("A network error occurred. DownloadFile will "
                              "use a cached version of the file")
                return self.local_filename
            else:
                raise ModuleError(
                        self.module,
                        "Network error: %s" % debug.format_exception(e))
        if response is None:
            return self.local_filename

        # Read response headers
        self.size_header = None
        if not self.read_headers(response):
            return self.local_filename

        # Download
        self.download(response)

        # Post download
        self.post_download(response)

        return self.local_filename

    def pre_download(self):
        pass

    def send_request(self):
        return self.opener.open(self.url)

    def read_headers(self, response):
        return True

    def download(self, response):
        try:
            dl_size = 0
            CHUNKSIZE = 4096
            f2 = open(self.local_filename, 'wb')
            while True:
                if self.size_header is not None:
                    self.module.logging.update_progress(
                            self.module,
                            dl_size * 1.0/self.size_header)
                chunk = response.read(CHUNKSIZE)
                if not chunk:
                    break
                dl_size += len(chunk)
                f2.write(chunk)
            f2.close()
            response.close()

        except Exception, e:
            try:
                os.unlink(self.local_filename)
            except OSError:
                pass
            raise ModuleError(
                    self.module,
                    "Error retrieving URL: %s" % debug.format_exception(e))

    def post_download(self, response):
        pass

    @property
    def is_in_local_cache(self):
        return os.path.isfile(self.local_filename)


class HTTPDownloader(Downloader):
    def pre_download(self):
        # Get ETag from disk
        try:
            with open(self.local_filename + '.etag') as etag_file:
                self.etag = etag_file.read()
        except IOError:
            self.etag = None

    def send_request(self):
        try:
            request = urllib2.Request(self.url)
            if self.etag is not None:
                request.add_header(
                    'If-None-Match',
                    self.etag)
            try:
                mtime = email.utils.formatdate(
                        os.path.getmtime(self.local_filename),
                        usegmt=True)
                request.add_header(
                    'If-Modified-Since',
                    mtime)
            except OSError:
                pass
            return self.opener.open(request)
        except urllib2.HTTPError, e:
            if e.code == 304:
                # Not modified
                return None
            raise

    def read_headers(self, response):
        try:
            self.mod_header = response.headers['last-modified']
        except KeyError:
            self.mod_header = None
        try:
            size_header = response.headers['content-length']
            if not size_header:
                raise ValueError
            self.size_header = int(size_header)
        except (KeyError, ValueError):
            self.size_header = None
        return True

    def _is_outdated(self):
        local_time = datetime.utcfromtimestamp(
                os.path.getmtime(self.local_filename))
        try:
            remote_time = strptime(self.mod_header,
                                   "%a, %d %b %Y %H:%M:%S %Z")
        except ValueError:
            try:
                remote_time = strptime(self.mod_header,
                                       "%a, %d %B %Y %H:%M:%S %Z")
            except ValueError:
                # unable to parse last-modified header, download file again
                debug.warning("Unable to parse Last-Modified header, "
                              "downloading file")
                return True
        return remote_time > local_time

    def download(self, response):
        if (not self.is_in_local_cache or
                not self.mod_header or self._is_outdated()):
            Downloader.download(self, response)

    def post_download(self, response):
        try:
            etag = response.headers['ETag']
        except KeyError:
            pass
        else:
            with open(self.local_filename + '.etag', 'w') as etag_file:
                etag = etag_file.write(etag)


class SSHDownloader(object):
    """ SSH downloader: downloads files via SCP, using paramiko and scp.

    Recognized URL schemes are:
        ssh://user[:password]@host[:port]/absolute/path
            Examples:
                ssh://john@vistrails.nyu.edu/home/john/example.txt
                ssh://eve:my%20secret@google.com/tmp/test%20file.bin
            Note that both password and path are url-encoded, that the path
            is absolute, and that the username must be specified
        scp://[user@]host:path
            Examples:
                scp://john@vistrails.nyu.edu:files/test.txt
                scp://poly.edu:/tmp/test.bin
            Note that nothing is url encoded, that the path can be relative
            (to the user's home directory) and that no username or port can
            be specified
    """

    SSH_FORMAT = re.compile(
            r'^'
            'ssh://'                    # Protocol
            '([A-Za-z0-9_/+.-]+)'       # 1 Username
            '(?::([^@]+))?'             # 2 Password
            '@([A-Za-z0-9_.-]+)'        # 3 Hostname
            '(?::([0-9]+))?'            # 4 Port number
            '(/.+)'                     # 5 Path (url-encoded!)
            '$'
            )
    SCP_FORMAT = re.compile(
            r'^'
            '(?:scp://)?'               # Protocol
            '(?:([A-Za-z0-9_/+.-]+)@)?' # 1 Username
            '([A-Za-z0-9_.-]+)'         # 2 Hostname
            ':(.+)'                     # 3 Path (not url-encoded)
            '$'
            )

    def __init__(self, url, module, insecure):
        self.url = url
        self.module = module

    def execute(self):
        # Parse URL
        password = None
        portnum = None
        if self.url.startswith('ssh:'):
            m = self.SSH_FORMAT.match(self.url)
            if m is None:
                raise ModuleError(self.module,
                                  "SSH error: invalid URL %r" % self.url)
            username, password, hostname, portnum, path = m.groups()
            password = urllib.unquote_plus(password)
            path = urllib.unquote_plus(path)
        elif self.url.startswith('scp:'):
            m = self.SCP_FORMAT.match(self.url)
            if m is None:
                raise ModuleError(self.module,
                                  "SSH error: invalid URL %r" % self.url)
            username, hostname, path = m.groups()
        else:
            raise ModuleError(self.module, "SSHDownloader: Invalid URL")

        if portnum is None:
            portnum = 22
        else:
            portnum = int(portnum)
        return self._open_ssh(username, password, hostname, portnum, path)

    def _open_ssh(self, username, password, hostname, portnum, path):
        paramiko = py_import('paramiko', {
                'pip': 'paramiko',
                'linux-debian': 'python-paramiko',
                'linux-ubuntu': 'python-paramiko',
                'linux-fedora': 'python-paramiko'})
        scp = py_import('scp', {
                'pip': 'scp'})

        local_filename = os.path.join(package_directory,
                                      urllib.quote_plus(self.url))

        ssh = paramiko.SSHClient()
        ssh.load_system_host_keys()
        try:
            ssh.connect(hostname, port=portnum,
                        username=username, password=password)
        except paramiko.SSHException, e:
            raise ModuleError(self.module, debug.format_exception(e))
        client = scp.SCPClient(ssh.get_transport())

        client.get(path, local_filename)
        return local_filename


downloaders = {
    'http': HTTPDownloader,
    'https': HTTPDownloader,
    'ssh': SSHDownloader,
    'scp': SSHDownloader}


class DownloadFile(Module):
    """ Downloads file from URL.

    This modules downloads a remote file. It tries to cache files on the local
    filesystem so as to not re-download unchanged files.

    Recognized URL schemes are:
        http://...
        https://...
        ftp://...
        ssh://user[:password]@host[:port]/absolute/path
            Examples:
                ssh://john@vistrails.nyu.edu/home/john/example.txt
                ssh://eve:my%20secret@google.com/tmp/test%20file.bin
            Note that both password and path are url-encoded, that the path
            is absolute, and that the username must be specified
        scp://[user@]host:path
            Examples:
                scp://john@vistrails.nyu.edu:files/test.txt
                scp://poly.edu:/tmp/test.bin
            Note that nothing is url encoded, that the path can be relative
            (to the user's home directory) and that no username or port can
            be specified
    """

    def compute(self):
        self.check_input('url')
        url = self.get_input('url')
        insecure = self.get_input('insecure')
        local_filename = self.download(url, insecure)
        self.set_output('local_filename', local_filename)
        result = PathObject(local_filename)
        self.set_output('file', result)

    def download(self, url, insecure):
        """ Tries to download a file from url.

        Returns the path to the local file.
        """
        scheme = urllib2.splittype(url)[0]
        DL = downloaders.get(scheme, Downloader)
        return DL(url, self, insecure).execute()


class HTTPDirectory(Module):
    """Downloads a whole directory recursively from a URL
    """

    def compute(self):
        self.check_input('url')
        url = self.get_input('url')
        insecure = self.get_input('insecure')
        local_path = self.download(url, insecure)
        self.set_output('local_path', local_path)
        local_dir = PathObject(local_path)
        self.set_output('directory', local_dir)

    def download(self, url, insecure):
        local_path = self.interpreter.filePool.create_directory(
                prefix='vt_http').name
        download_directory(url, local_path, insecure)
        return local_path


class RepoSync(Module):
    """ VisTrails Server version of RepoSync modules. Customized to play
    nicely with crowdlabs. Needs refactoring.

    RepoSync enables data to be synced with a online repository. The designated file
    parameter will be uploaded to the repository on execution,
    creating a new pipeline version that links to online repository data.
    If the local file isn't available, then the online repository data is used.
    """

    def __init__(self):
        Module.__init__(self)

        config = get_vistrails_persistent_configuration()
        if config.check('webRepositoryURL'):
            self.base_url = config.webRepositoryURL
        else:
            raise ModuleError(self,
                              ("No webRepositoryURL value defined"
                               " in the Expert Configuration"))

        # check if we are running in server mode
        # this effects how the compute method functions
        if config.check('isInServerMode'):
            self.is_server = bool(config.isInServerMode)
        else:
            self.is_server = False

        # TODO: this '/' check should probably be done in core/configuration.py
        if self.base_url[-1] == '/':
            self.base_url = self.base_url[:-1]

    # used for invaliding cache when user isn't logged in to crowdLabs
    # but wants to upload data
    def invalidate_cache(self):
        return False

    def validate_cache(self):
        return True

    def _file_is_in_local_cache(self, local_filename):
        return os.path.isfile(local_filename)

    def checksum_lookup(self):
        """ checks if the repository has the wanted data """

        checksum_url = "%s/datasets/exists/%s/" % (self.base_url,
                                                   self.checksum)
        self.on_server = False
        try:
            check_dataset_on_repo = urllib2.urlopen(url=checksum_url)
            self.up_to_date = True if \
                    check_dataset_on_repo.read() == 'uptodate' else False
            self.on_server = True
        except urllib2.HTTPError:
            self.up_to_date = True

    def data_sync(self):
        """ downloads/uploads/uses the local file depending on availability """
        self.checksum_lookup()

        # local file not on repository, so upload
        if not self.on_server and os.path.isfile(self.in_file.name):
            cookiejar = vistrails.gui.repository.QRepositoryDialog.cookiejar
            if cookiejar:
                register_openers(cookiejar=cookiejar)

                params = {'dataset_file': open(self.in_file.name, 'rb'),
                          'name': self.in_file.name.split('/')[-1],
                          'origin': 'vistrails',
                          'checksum': self.checksum}

                upload_url = "%s/datasets/upload/" % self.base_url

                datagen, headers = multipart_encode(params)
                request = urllib2.Request(upload_url, datagen, headers)
                try:
                    result = urllib2.urlopen(request)
                    if result.code != 200:
                        show_warning("Upload Failure",
                                     "Data failed to upload to repository")
                        # make temporarily uncachable
                        self.is_cacheable = self.invalidate_cache
                    else:
                        debug.warning("Push to repository was successful")
                        # make sure module caches
                        self.is_cacheable = self.validate_cache
                except Exception, e:
                    show_warning("Upload Failure",
                                 "Data failed to upload to repository")
                    # make temporarily uncachable
                    self.is_cacheable = self.invalidate_cache
                debug.warning('RepoSync uploaded %s to the repository' % \
                              self.in_file.name)
            else:
                show_warning("Please login", ("You must be logged into the web"
                                              " repository in order to upload "
                                              "data. No data was synced"))
                # make temporarily uncachable
                self.is_cacheable = self.invalidate_cache

            # use local data
            self.set_output("file", self.in_file)
        else:
            # file on repository mirrors local file, so use local file
            if self.up_to_date and os.path.isfile(self.in_file.name):
                self.set_output("file", self.in_file)
            else:
                # local file not present or out of date, download or use cache
                self.url = "%s/datasets/download/%s" % (self.base_url,
                                                       self.checksum)
                local_filename = package_directory + '/' + \
                        urllib.quote_plus(self.url)
                if not self._file_is_in_local_cache(local_filename):
                    # file not in cache, download.
                    try:
                        urllib.urlretrieve(self.url, local_filename)
                    except IOError, e:
                        raise ModuleError(self, ("Invalid URL: %s" % e))
                out_file = PathObject(local_filename)
                debug.warning('RepoSync is using repository data')
                self.set_output("file", out_file)


    def compute(self):
        # if server, grab local file using checksum id
        if self.is_server:
            self.check_input('checksum')
            self.checksum = self.get_input("checksum")
            # get file path
            path_url = "%s/datasets/path/%s/"%(self.base_url, self.checksum)
            dataset_path_request = urllib2.urlopen(url=path_url)
            dataset_path = dataset_path_request.read()

            if os.path.isfile(dataset_path):
                out_file = PathObject(dataset_path)
                self.set_output("file", out_file)
        else: # is client
            self.check_input('file')
            self.in_file = self.get_input("file")
            if os.path.isfile(self.in_file.name):
                # do size check
                size = os.path.getsize(self.in_file.name)
                if size > 26214400:
                    show_warning("File is too large",
                                 "file is larger than 25MB, "
                                 "unable to sync with web repository")
                    self.set_output("file", self.in_file)
                else:
                    # compute checksum
                    f = open(self.in_file.name, 'r')
                    self.checksum = hashlib.sha1()
                    block = 1
                    while block:
                        block = f.read(128)
                        self.checksum.update(block)
                    f.close()
                    self.checksum = self.checksum.hexdigest()

                    # upload/download file
                    self.data_sync()

                    # set checksum param in module
                    if not self.has_input('checksum'):
                        self.change_parameter('checksum', [self.checksum])

            else:
                # local file not present
                if self.has_input('checksum'):
                    self.checksum = self.get_input("checksum")

                    # download file
                    self.data_sync()


class URLEncode(Module):
    def compute(self):
        value = self.get_input('string')
        self.set_output('encoded', urllib.quote_plus(value))


class URLDecode(Module):
    def compute(self):
        encoded = self.get_input('encoded')
        self.set_output('string', urllib.unquote_plus(encoded))


def initialize(*args, **keywords):
    reg = vistrails.core.modules.module_registry.get_module_registry()
    basic = vistrails.core.modules.basic_modules

    reg.add_module(DownloadFile)
    reg.add_input_port(DownloadFile, "url", (basic.String, 'URL'))
    reg.add_input_port(DownloadFile, 'insecure',
                       (basic.Boolean, "Allow invalid SSL certificates"),
                       optional=True, defaults="['False']")
    reg.add_output_port(DownloadFile, "file",
                        (basic.File, 'local File object'))
    reg.add_output_port(DownloadFile, "local_filename",
                        (basic.String, 'local filename'), optional=True)

    reg.add_module(HTTPDirectory)
    reg.add_input_port(HTTPDirectory, 'url', (basic.String, "URL"))
    reg.add_input_port(HTTPDirectory, 'insecure',
                       (basic.Boolean, "Allow invalid SSL certificates"),
                       optional=True, defaults="['False']")
    reg.add_output_port(HTTPDirectory, 'directory',
                        (basic.Directory, "local Directory object"))
    reg.add_output_port(HTTPDirectory, 'local_path',
                        (basic.String, "local path"), optional=True)

    reg.add_module(RepoSync)
    reg.add_input_port(RepoSync, "file", (basic.File, 'File'))
    reg.add_input_port(RepoSync, "checksum",
                       (basic.String, 'Checksum'), optional=True)
    reg.add_output_port(RepoSync, "file", (basic.File,
                                           'Repository Synced File object'))
    reg.add_output_port(RepoSync, "checksum",
                        (basic.String, 'Checksum'), optional=True)

    reg.add_module(URLEncode)
    reg.add_input_port(URLEncode, "string", basic.String)
    reg.add_output_port(URLEncode, "encoded", basic.String)

    reg.add_module(URLDecode)
    reg.add_input_port(URLDecode, "encoded", basic.String)
    reg.add_output_port(URLDecode, "string", basic.String)

    global package_directory
    dotVistrails = current_dot_vistrails()
    package_directory = os.path.join(dotVistrails, "HTTP")

    if not os.path.isdir(package_directory):
        try:
            debug.log("Creating HTTP package directory: %s" % package_directory)
            os.mkdir(package_directory)
        except Exception, e:
            raise RuntimeError("Failed to create cache directory: %s" %
                               package_directory, e)


def handle_module_upgrade_request(controller, module_id, pipeline):
    module_remap = {
            # HTTPFile was renamed DownloadFile
            'HTTPFile': [
                (None, '1.0.0', 'DownloadFile', {})
            ],
        }

    return UpgradeWorkflowHandler.remap_module(controller,
                                               module_id,
                                               pipeline,
                                               module_remap)


###############################################################################

import unittest


class TestDownloadFile(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        from vistrails.core.packagemanager import get_package_manager
        from vistrails.core.modules.module_registry import MissingPackage
        pm = get_package_manager()
        try:
            pm.get_package('org.vistrails.vistrails.http')
        except MissingPackage:
            pm.late_enable_package('URL')

    def testIncorrectURL(self):
        from vistrails.tests.utils import execute
        self.assertTrue(execute([
                ('DownloadFile', identifier, [
                    ('url', [('String', 'http://idbetthisdoesnotexistohrly')]),
                ]),
            ]))

    def testIncorrectURL_2(self):
        from vistrails.tests.utils import execute
        self.assertTrue(execute([
                ('DownloadFile', identifier, [
                    ('url', [('String', 'http://neitherodesthisohrly')]),
                ]),
            ]))


class TestHTTPDirectory(unittest.TestCase):
    def test_download(self):
        url = 'http://www.vistrails.org/testing/httpdirectory/test/'

        import shutil
        import tempfile
        testdir = tempfile.mkdtemp(prefix='vt_test_http_')
        try:
            download_directory(url, testdir)
            files = {}
            def addfiles(dirpath):
                td = os.path.join(testdir, dirpath)
                for name in os.listdir(td):
                    filename = os.path.join(testdir, dirpath, name)
                    dn = os.path.join(dirpath, name)
                    if os.path.isdir(filename):
                        addfiles(os.path.join(dirpath, name))
                    else:
                        with open(filename, 'rb') as f:
                            files[dn.replace(os.sep, '/')] = f.read()
            addfiles('')
            self.assertEqual(len(files), 4)
            del files['f.html']
            self.assertEqual(files, {
                    'a': 'aa\n',
                    'bb': 'bb\n',
                    'cc/d': 'dd\n',
                })
        finally:
            shutil.rmtree(testdir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# This example demonstrates how to use multiple renderers within a
# render window. It is a variation of the Cone.py example. Please
# refer to that example for additional documentation.
#

import vtk
import time

# 
# Next we create an instance of vtkConeSource and set some of its
# properties. The instance of vtkConeSource "cone" is part of a visualization
# pipeline (it is a source process object); it produces data (output type is
# vtkPolyData) which other filters may process.
#
cone = vtk.vtkConeSource()
cone.SetHeight( 3.0 )
cone.SetRadius( 1.0 )
cone.SetResolution( 10 )

# 
# In this example we terminate the pipeline with a mapper process object.
# (Intermediate filters such as vtkShrinkPolyData could be inserted in
# between the source and the mapper.)  We create an instance of
# vtkPolyDataMapper to map the polygonal data into graphics primitives. We
# connect the output of the cone souece to the input of this mapper.
#
coneMapper = vtk.vtkPolyDataMapper()
coneMapper.SetInputConnection(cone.GetOutputPort())

# 
# Create an actor to represent the cone. The actor orchestrates rendering of
# the mapper's graphics primitives. An actor also refers to properties via a
# vtkProperty instance, and includes an internal transformation matrix. We
# set this actor's mapper to be coneMapper which we created above.
#
coneActor = vtk.vtkActor()
coneActor.SetMapper(coneMapper)

# 
# Create two renderers and assign actors to them. A renderer renders into a
# viewport within the vtkRenderWindow. It is part or all of a window on the
# screen and it is responsible for drawing the actors it has.  We also set
# the background color here. In this example we are adding the same actor
# to two different renderers; it is okay to add different actors to
# different renderers as well.
#
ren1 = vtk.vtkRenderer()
ren1.AddActor(coneActor)
ren1.SetBackground(0.1, 0.2, 0.4)
ren1.SetViewport(0.0, 0.0, 0.5, 1.0)

ren2 = vtk.vtkRenderer()
ren2.AddActor(coneActor)
ren2.SetBackground(0.1, 0.2, 0.4)
ren2.SetViewport(0.5, 0.0, 1.0, 1.0)

#
# Finally we create the render window which will show up on the screen.
# We add our two renderers into the render window using AddRenderer. We also
# set the size to be 600 pixels by 300.
#
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer( ren1 )
renWin.AddRenderer( ren2 )
renWin.SetSize(600, 300)

#
# Make one camera view 90 degrees from other.
#
ren1.ResetCamera()
ren1.GetActiveCamera().Azimuth(90)

#
# Now we loop over 360 degreeees and render the cone each time.
#
# for i in range(0,360):
#     time.sleep(0.03)
    
#     renWin.Render()
#     ren1.GetActiveCamera().Azimuth( 1 )
#     ren2.GetActiveCamera().Azimuth( 1 )



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

# https://gist.github.com/remram44/6540454

from __future__ import division

from HTMLParser import HTMLParser
import os
import re

from .https_if_available import build_opener


re_url = re.compile(r'^(([a-zA-Z_-]+)://([^/]+))(/.*)?$')

def resolve_link(link, url):
    m = re_url.match(link)
    if m is not None:
        if not m.group(4):
            # http://domain -> http://domain/
            return link + '/'
        else:
            return link
    elif link[0] == '/':
        # /some/path
        murl = re_url.match(url)
        return murl.group(1) + link
    else:
        # relative/path
        if url[-1] == '/':
            return url + link
        else:
            return url + '/' + link


class ListingParser(HTMLParser):
    """Parses an HTML file and build a list of links.

    Links are stored into the 'links' set. They are resolved into absolute
    links.
    """
    def __init__(self, url):
        HTMLParser.__init__(self)

        if url[-1] != '/':
            url += '/'
        self.__url = url
        self.links = set()

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            for key, value in attrs:
                if key == 'href':
                    if not value:
                        continue
                    value = resolve_link(value, self.__url)
                    self.links.add(value)
                    break


def download_directory(url, target, insecure=False):
    def mkdir():
        if not mkdir.done:
            try:
                os.mkdir(target)
            except OSError:
                pass
            mkdir.done = True
    mkdir.done = False

    opener = build_opener(insecure=insecure)
    response = opener.open(url)

    if response.info().type == 'text/html':
        contents = response.read()

        parser = ListingParser(url)
        parser.feed(contents)
        for link in parser.links:
            link = resolve_link(link, url)
            if link[-1] == '/':
                link = link[:-1]
            if not link.startswith(url):
                continue
            name = link.rsplit('/', 1)[1]
            if '?' in name:
                continue
            mkdir()
            download_directory(link, os.path.join(target, name), insecure)
        if not mkdir.done:
            # We didn't find anything to write inside this directory
            # Maybe it's a HTML file?
            if url[-1] != '/':
                end = target[-5:].lower()
                if not (end.endswith('.htm') or end.endswith('.html')):
                    target = target + '.html'
                with open(target, 'wb') as fp:
                    fp.write(contents)
    else:
        buffer_size = 4096
        with open(target, 'wb') as fp:
            chunk = response.read(buffer_size)
            while chunk:
                fp.write(chunk)
                chunk = response.read(buffer_size)


###############################################################################

import unittest


class TestLinkResolution(unittest.TestCase):
    def test_absolute_link(self):
        self.assertEqual(
                resolve_link('http://website.org/p/test.txt',
                             'http://some/other/url'),
                'http://website.org/p/test.txt')
        self.assertEqual(
                resolve_link('http://website.org',
                             'http://some/other/url'),
                'http://website.org/')

    def test_absolute_path(self):
        self.assertEqual(
                resolve_link('/p/test.txt', 'http://some/url'),
                'http://some/p/test.txt')
        self.assertEqual(
                resolve_link('/p/test.txt', 'http://some/url/'),
                'http://some/p/test.txt')
        self.assertEqual(
                resolve_link('/p/test.txt', 'http://site'),
                'http://site/p/test.txt')
        self.assertEqual(
                resolve_link('/p/test.txt', 'http://site/'),
                'http://site/p/test.txt')

    def test_relative_path(self):
        self.assertEqual(
                resolve_link('some/file', 'http://site/folder'),
                'http://site/folder/some/file')
        self.assertEqual(
                resolve_link('some/file', 'http://site/folder/'),
                'http://site/folder/some/file')
        self.assertEqual(
                resolve_link('some/dir/', 'http://site/folder'),
                'http://site/folder/some/dir/')


class TestParser(unittest.TestCase):
    def test_parse(self):
        parser = ListingParser('http://a.remram.fr/test')
        parser.feed("""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><html><head><title>
Index of /test</title></head><body><h1>Index of /test</h1><table><tr><th>
<img src="/icons/blank.gif" alt="[ICO]"></th><th><a href="?C=N;O=D">Name</a>
</th><th><a href="?C=M;O=A">Last modified</a></th><th><a href="?C=S;O=A">Size
</a></th><th><a href="?C=D;O=A">Description</a></th></tr><tr><th colspan="5">
<hr></th></tr><tr><td valign="top"><img src="/icons/back.gif" alt="[DIR]"></td>
<td><a href="/">Parent Directory</a></td><td>&nbsp;</td><td align="right">  - 
</td><td>&nbsp;</td></tr><tr><td valign="top">
<img src="/icons/unknown.gif" alt="[   ]"></td><td><a href="a">a</a></td>
<td align="right">11-Sep-2013 15:46  </td><td align="right">  3 </td><td>&nbsp;
</td></tr><tr><td valign="top"><img src="/icons/unknown.gif" alt="[   ]"></td>
<td><a href="/bb">bb</a></td><td align="right">11-Sep-2013 15:46  </td>
<td align="right">  3 </td><td>&nbsp;</td></tr><tr><td valign="top">
<img src="/icons/folder.gif" alt="[DIR]"></td><td><a href="/cc/">cc/</a></td>
<td align="right">11-Sep-2013 15:46  </td><td align="right">  - </td><td>&nbsp;
</td></tr><tr><td valign="top"><img src="/icons/folder.gif" alt="[DIR]"></td>
<td><a href="http://a.remram.fr/dd">dd/</a></td><td align="right">
11-Sep-2013 15:46  </td><td align="right">  - </td><td>&nbsp;</td></tr><tr>
<th colspan="5"><hr></th></tr></table></body></html>
        """)
        links = set(l for l in parser.links if '?' not in l)
        self.assertEqual(links, set([
                'http://a.remram.fr/',
                'http://a.remram.fr/test/a',
                'http://a.remram.fr/bb',
                'http://a.remram.fr/cc/',
                'http://a.remram.fr/dd',
        ]))



#!/usr/bin/env python

# This example demonstrates the use of vtkSTLReader to load data into
# VTK from a file.  This example also uses vtkLODActor which changes
# its graphical representation of the data to maintain interactive
# performance.

import vtk
from vtk.util.misc import vtkGetDataRoot
VTK_DATA_ROOT = vtkGetDataRoot()

# Create the reader and read a data file.  Connect the mapper and
# actor.
sr = vtk.vtkSTLReader()
sr.SetFileName(VTK_DATA_ROOT + "/Data/42400-IDGH.stl")

stlMapper = vtk.vtkPolyDataMapper()
stlMapper.SetInputConnection(sr.GetOutputPort())

stlActor = vtk.vtkLODActor()
stlActor.SetMapper(stlMapper)

# Create the Renderer, RenderWindow, and RenderWindowInteractor
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Add the actors to the render; set the background and size
ren.AddActor(stlActor)
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(500, 500)

# Zoom in closer
ren.ResetCamera()
cam1 = ren.GetActiveCamera()
cam1.Zoom(1.4)

iren.Initialize()
renWin.Render()
iren.Start()
